﻿/*
	@version 3.7.1
	@author Jobe Makar, Electrotank, Inc
	@description This class is used to for communication between a Flash client and ElectroServer.
				For more information or help using ElectroServer, please visit http://www.electro-server.com
*/
class ElectroServer extends XMLSocket {
	//@exclude
	//Last edit: 1/23/2006
	//
	//Started: 8/15/03
	//
	//-----Revision History---------
	//	9/18/03		Officially released
	//	9/30/03		Added version detection. If class connects to an older ElectroServer 3 version then an error appears
	//	10/8/03		Fixed createRoom() auto_join bug. Now behaves properly
	//	10/11/03	Fixed chat message bug. A pure number chat message was being converted to a number on reentry
	//	10/20/03	Changed the way variables are parsed in received plugin message
	//	11/5/03		Fixed a case sensitive issue with room variables
	//	11/5/03		Fixed a sendMove/moveReceived issue
	//	11/7/03		Added FloodingFilterEnabled support
	//	11/12/03	In createGameRoom I added support of either description or Description, case insensitive
	//	12/14/03	Added the ability to receive an array of variables in the loggedIn event
	//	12/16/03	Added setModeratorLabel(true|false, "[Mod]") -- If someone is a moderator then their label gets "[Mod]" appended
	//	1/5/04		Fixed default zone name issue
	//	1/19/04		Fixed user variable support issues
	//	2/1/04		Added getLoggedInUserCount(). Requests the total number of users logged into the server in all zones
	//	2/1/04		Added getServerTime(). Returns the number of miliseconds since 1970.
	//	2/12/04		getRoomsInZone('zone name') added. It is there for clients that receive higher loads of bandwidth. It triggers onRoomsInZoneLoaded
	//	2/23/04		Fixed a WDDX GetRoomsInZone parsing bug
	//	2/26/04		Added a hidden room bug work around to class.
	//	5/6/04		Added event for the response of getLoggedInUserCount. It is es.loggedInUserCountUpdated(num_logged_in)
	//	5/22/04		Added an event to capture the getServerTime method results. onGetServerTime
	//	8/8/04		Fixed a bug that caused the userlist to show incorrect results
	//	8/15/04		Added rawMessageReceived()
	//	8/28/04		Fixed userVariable client-side data saving bug
	//	9/26/2004	Added buddy stuff. es.addBuddy("jobem"), es.removeBuddy("jobem"), es.buddyLoggedIn event, es.buddyLoggedOut event
	//	9/26/2004	Added getUserLocation("jobem") es.userLocationLoaded event is fired when loaded. username, zone, and room are passed in as a string
	//	9/26/2004	Added getUsersInRoom("zone name", "room name"). es.usersInRoomLoaded event is fired. An array of objects with a username property is returned
	//	10/15/2004	Enhanced usersInRoomLoaded event. It now also returns the room and zone names
	//  11/10/2004	Fixed a uservariable update bug.
	//	11/11/2004  Fixed a createGameRoom bug where updatable was always false.
	//	12/15/2004	Added a 4th param to the userListUpdated event, it is a reference to the user object
	//	1/13/2005	Added an optional 3rd parameter to the login(name, pass, variables) method, 'variables'. It is an object that can contain name value pairs, captured by the login event handler on the server
	//	1/20/2005	Fixed userLocationLoaded bug fixed
	//	2/27/2005	Added registerRawHandler	
	//	2/27/2005 	Added sendRaw(raw_msg)
	//	4/10/2005 	Fixed a bug where a uservariable wasn't being fully deleted
	//	5/5/2005	Added userListEnabled=true/false property to the room object at creation time.
	//	11/7/2005	Fixed bug where connect() method didn't accept ip and port. connect(ip, port) now works.
	//	11/9/2005	Bug fixed. onConnection was not returning 'false' or an error when a socket connection failed. 
	//	11/22/2005	New feature: es.logout(true/false)  If the paramter is true then you are logged out and disconnected. If false, then you are logged out and retain your conneciton.
	//	12/18/2005	Rare bug fixed related to WDDX and room creation.
	//	1/16/2006	Put all documentation inside of this class
	//
	//!!! Download ElectroServer class updates from here: http://www.electro-server.com/downloads.aspx
	//
	//=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+=~=+
	//Events
	public var ignore;
	
	/*
		Event fired as a result of the {@link #login} method
		@description When use use the {@link #login} method to attempt a login, this event is eventually fired.<br><br>
					<b>Event parameters</b><br>
					<i>success</i> (Boolean) Value is true if the login is a success.<br>
					<i>error</i>  (String) Value exists if success = false.
		@example
			See {@link #login} for usage.
	*/
	public var loggedIn:Function;
	
	/*
		Event fired as a result of the {@link #connect} method
		@description When you use the {@link #connect} method a connection is attempted with ElectroServer. When the connection has
					either succeeded or failed this event is fired.<br><br>
					<b>Event parameters</b><br>
					<i>success</i> (Boolean) Value is true if the connection is a success.<br>
					<i>error</i>  (String) Value exists if success = false. It contains a reason why the connection failed.
		@example 
				<code>
					function connectionHandler(success:Boolean, error:String) {
						if (success) {
							gotoAndStop("Login");
						} else {
							msg.text = error;
							trace(error);
							gotoAndStop("Error");
						}
					}
					es.onConnection = connectionHandler;
				</code>
	*/
	public var onConnection:Function;
	
	/*
		Event fired when you change zones
		@description Event fired when you enter a new zone. Simply gives you the name of the zone that you have just entered.<br><br>
					A zone is a collection of rooms much like a room is a collection of users. When in a zone you get 
					information on the other rooms in your zone. You have no knowledge of any of the other zones.<br><br>
					<b>Event parameters</b><br>
					<i>name</i> The name of the newly joined zone.
		@example
				<code>
					function zoneChanged(name:String) {
						trace("I just entered this zone: "+name);
					}
					es.zoneChanged = zoneChanged;
				</code>
	*/
	public var zoneChanged:Function;
	
	/*
		Event fired when the room list changes
		@description Event fired when an update to the zone roomlist is received.<br>
					If updateType is "all": This is the initial roomlist received when joining a new zone
					(or logging in for the first time).<br>
					If updateType is "roomupdated": This is just a change in details for one room, such as its description,
					the number of users in that room, etc.<br>
					If updateType is "roomcreated": The room was just created.<br>
					If updateType is "roomdeleted": A room was jut removed. The 'roomReference' parameter is a string representing
					the name of the room just deleted. It is not a reference to the room object since it has been deleted.<br>
					Note: The reference to the room list can also be retrieved by using the getRoomList() method.<br><br>
					<b>Event parameters</b><br>
					<i>roomList</i> An array of objects describing the list of rooms. (Array)<br>
					<i>updateType</i> "all", "roomupdated", "roomcreated", "roomdeleted" (String)<br>
					<i>roomReference</i> Reference to the room objec that was updated. (Object)
		@example 
				The following is code that could be used to update a visual list of rooms 
				(perhaps in a list box component) as updates come in:
				<code>
					function roomListUpdated(roomList:Array, updateType:String, roomReference:Object) {
						if (updateType == "all") {
							//The entire list is new
							showRooms(roomList);
						} else if (updateType == "roomupdated") {
							//A room that exists was updated
							updateRoomInList(roomReference);
						} else if (updateType == "roomcreated") {
							//A new room was added
							addRoomToList(roomReference);
						} else if (updateType == "roomdeleted") {
							//A room was removed
							removeRoomFromList(roomReference);
						}
					}
					es.roomListUpdated = roomListUpdated;
				</code>

	*/
	public var roomListUpdated:Function;
	
	/*
		Event fired when the room variables in your room change
		@description Event fired when a room variable is created, modified, or removed.
					See {@link #createRoomVariable}, {@link #deleteRoomVariable}<br><br>
					Room variables are variables actually stored on the server in your room. They can be created, modified,
					or removed. By default, a room variable can be modified and will automatically be removed when the user
					that created it leaves the room. However, a room variable can be flagged to be persistent which means
					that it will nto delete when the user that creates it leaves the room. Also, a room variable can be
					flagged as locked, which means that once locked it can no longer be modified (but can still be deleted).<br><br>
					<b>Event parameters</b><br>
					<i>type</i> "all", "updated", "created", "deleted" (Object)<br>
					<i>roomVariables</i> An object containing all of the room variables. (Object)<br>
					<i>name</i> String name of the variable updated. (blank if type=="all") (Object)<br>
		@example 
				If you created a chat room where you wanted to put an icon next to users that say they want to chat one-on-one
				with someone, then room variables can help there. Someone might create a room variable with their username and
				the following code is how it would be handled:
				<code>
					function roomVariablesUpdated(type:String, ob:Object, name:String) {
						if (type == "updated" || type == "created") {
							flagUserAsWantingToChat(name);
						}
					}
					es.roomVariablesUpdated = roomVariablesUpdated;
				</code>
				In the {@link #createRoomVariable} method documention there is code to show a variable being created showing if a secret door has been unlocked or not. Here is that code again followed by the code that would intercept this variable once created:
				<br>To create it:
				<code>
					var ob:Object = new Object();
					ob.name = "secretDoorUnlocked";
					ob.data = "true";
					ob.locked = true;
					ob.persistent = true;
					es.createRoomVariable(ob);
					//To intercept it
					function roomVariablesUpdated(type:String, ob:Object, name:String) {
						if (type == "updated") {
							if (name == "secretDoorUnlocked") {
								toggleSecretDoor(ob.secretDoorUnlocked);
							}
						}
					}
					es.roomVariablesUpdated = roomVariablesUpdated;
				</code>
	*/
	public var roomVariablesUpdated:Function;
	
	/*
		Event fired as a result of the {@link #createRoom} or {@link #joinRoom} methods
		@description Event fired when a you have been accepted or denied access to a room. When you invoke the {@link #createRoom}
					method or the {@link #joinRoom} method you will receive a roomJoined event as a response after the server
					processes your request. This event passes in an object with a success = true or false property. If success = false
					then this object also contains an error message.<br><br>
					Typical errors are due to invalid room passwords (when trying to join a password protected room), 
					invalid room name, or room full.<br><br>
					Note: the reference to the room that you are in can also be retrieved by using the {@link #getRoom} method.<br><br>
					<b>Event parameters</b><br>
					<i>results</i> An ojbect containing 'success' boolean of true or false, and an 'error' property if needed (Object)<br>
					<i>myRoom</i> A reference to the object describing your room (Object)
		@example 
				<code>
					function roomJoined(results:Object, myRoom:Object) {
						if (results.success) {
							//Joined the room
							gotoAndStop("Chat");
						} else {
							//Did not join the room
							trace(results.error);
							msg.text = results.error;
							gotoAndStop("Error");
						}
					}
					es.roomJoined = roomJoined;
				</code>
	*/
	public var roomJoined:Function;
	
	/*
		Event fired when the list of users in your room updates
		@description Event fired when the users in your room changes. When you first join a room you get the full user list and
					type is "all". After that you only receive "userjoined" and "userleft" updates.<br><br>
					If type is "all" then the userRefrence parameter is blank. If type is "userjoined" then the userReference 
					parameter contains a reference to the user object. If type is "userleft" then the userReference parameter 
					is a string name of the user that left (not a reference to the object as the object has been removed).
					<br>See also {@link #getUserList}<br><br>
					<b>Event parameters</b><br>
					<i>userList</i> Array of user objects (Array)<br>
					<i>type</i> "all", "userjoined", "userleft" (String)<br>
					<i>userReference</i> Name of the user that joined or left (String)
		@example 
			<code>
				function userListUpdated(userList:Array, type:String, userReference:Object) {
					if (type == "all") {
						//The entire list is new
						buildUserList(userList);
					} else if (type == "userjoined") {
						//A user joined the room
						addUserToList(userReference);
					} else if (type == "userleft") {
						//A user left the room
						removeUserFromList(userReference);
					}
				}
				es.userListUpdated = userListUpdated;
			</code>
	*/
	public var userListUpdated:Function;
	
	/*
		Event fired when you receive a public or private message
		@description This is the most common way to receive chat messages.<br>
					<b>Event parameters</b><br>
					<i>type</i> Either "public" or "private" (String)<br>
					<i>message</i> The message received (String)<br>
					<i>from</i> The name of the user that sent the message (String)<br>
					<i>variables</i> An object containing the message variables sent, if any (Object)
		@example 
				<code>
					function gotMessage(type:String, message:String, from:String) {
						if (type == "public") {
							chatHistory += from+": "+message;
						} else if (tyep == "private") {
							showPrivateMessagePopUp(message, from);
						}
					}
					es.messageReceived = gotMessage;
				</code>
	*/
	public var messageReceived:Function;
	
	/*
		Event fired when you receive a public or private move from another player
		@description Users can send each other objects to make game creation much easier. See {@link #sendMove}.<br><br>
					<b>Event parameters</b><br>
					<i>type</i> Either "public" or "private" (Object)<br>
					<i>object</i> An object sent to you (Object)</br>
					<i>from</i> The name of the user that sent the move (Object)<br>
		@example 
				The following is an example of a move that could be seen in a game where characters can jump and shoot.
				<code>
					function moveReceived(type:String, ob:Object, from:String) {
						if (type == "private") {
							var action:String = ob.action;
							if (action == "shoot") {
								var x:Number = ob.x;
								var y:Number = ob.y;
								var angle:Number = ob.angle;
								var speed:Number = ob.speed;
								createProjectile(x, y, angle, speed);
							} else if (action == "jump") {
								makeOpponentJump();
							}
						}
					}
					es.moveReceived = moveReceived;
				</code>

	*/
	public var moveReceived:Function;
	
	/*
		Event fired when the AssignedNumber.value property on the users in your room changes
		@description Event fired when the users in your room have been renumbered. By default, when creating a room the Numbered
					attribute is 'false', meaning the users in that room are not numbered. However, when Numbered is 'true', the 
					users in the room are numbered.<br><br>
					<b>Why numbered users can be usefull</b><br>
					When creating multiplayer games that use more than two players it is quite a task to keep track of the order
					of users in the game. What happens if someone leaves the game in the middle and it is their turn to move?
					Due to Internet latency and the various scenarios by which a person can leave a game it is difficult to guarantee 
					no problems.<br><br>
					With numbered users you always know the order of the users. If it is number 3's turn to move and then number 3 
					leaves the room, well then it is still number 3's turn because number 4 just shifted down a position. 
					Since this is kept on the server there is no chance of synchronization problems.<br><br>
					This concept is not the easiest to understand initially, it may help to view sample FLA files.<br><br>
					Note: the userList is passed in to the event handler. The userList can also be retrieved using the 
					{@link #getUserList} method.<br><br>
					Note: if you use server-side plugins then you can achieve the same benefits much more easily. The benefits
							being a centralized location that knows whose turn it is.<br><br>
					<b>Event parameters</b><br>
					<i>userList Array of user objects. (Array)</i>
		@example 
				<code>
					function usersRenumbered(userList:Array) {
						for (var i:Number = 0; i < userList.length; ++i) {
							trace(userList[i].AssignedNumber.value);
						}
					}
					es.usersRenumbered = usersRenumbered;
				</code>
	*/
	public var usersRenumbered:Function;
	
	/*
		Event fired when the client receives a message sent from a plugin 
		@description Event fired when Flash receives a message from a plugin.<br><br>
					ElectroServer 3 has the ability to run custom-written plugins created using either Java or ActionScript.
					These plugins can send messages to individual users or to a room. When a message is received by a Flash client
					from a plugin, the name of the plugin is passed in. Also, an action name is passed in. This can be something 
					like "CreateMonster". The plugin can also send an unlimited number of variables, which are found in the
					variables parameter.<br><br>
					Note: Please read ElectroServer 3 documentation on writing server plugins.<br><br>
					<b>Event parameters</b><br>
					<i>plugin</i> The name of the plugin that sent the message (String)<br>
					<i>action</i> Action that the plugin wants you to take (String)<br>
					<i>variables</i> Object holding variables sent by the plugin (Object)<br>
		@example 
				What follows is a snippet of code that could be straight out of a real-time RPG controlled by a server-side plugin
				<code>
					function pluginMessageReceived(plugin:String, action:String, variables:Object) {
						if (plugin == "RPG_Monster_Generator_Plugin") {
							if (action == "Spawn_New_Bad_Guy") {
								var badGuyType = variables.badGuyType;
								var x:Number = variables.x;
								var y:Number = variables.y;
								var armor:String = variables.armor;
								var speed:Number = variables.speed;
								var ferocity:Number = variables.ferocity;
								addNewBadGuyToMap(x, y, badGuyType, armor, speed, ferocity);
							} else if (action == "Picked_Up_Item") {
								var item:String = variables.item;
								var worth:Number = variables.worth;
								addItemToInventory(item, worth);
							}
						}
					}
					es.pluginMessageReceived = pluginMessageReceived;
				</code>

	*/
	public var pluginMessageReceived:Function;
	
	/*
		Event fired when the number of users in the zone changes
		@description Event fired when the number of users in the zone changes. Simply gives you the number of users in your
					zone when ever that number is changed.<br><br>
					<b>Event parameters</b><br>
					<i>numUsers</i> Number of users in the zone
		@example 
				<code>
					function zoneUpdated (numUsers:Number) {
						trace("The number of people in my zone is: "+numUsers);
					}
					es.zoneUpdated = zoneUpdated;
				</code>
	*/
	public var zoneUpdated:Function;
	
	/*
		Event fired when the configuration file has been loaded as a result of {@link #loadConfiguration}.
		@description Event fired when the configuration file has finished loading. The configuration XML file is passed 
					into the callback function. See the provided admin client for example usage.<br><br>
					<b>Event parameters</b><br>
					<i>ob</i> Object containing the parse configuration file.
		@example 
				<code>
					es.configurationLoaded = function(ob:Object) {
						//config loaded
					}
					es.loadConfiguration();
				</code>
	*/
	public var configurationLoaded:Function;
	
	/*
		Event fired when the connection to ElectroServer closes.
		@description Event fired when connection to the server is lost. If 'true' then the connection was closed due
					to forces out side of the Flash client (like ElectroServer closing it or a broken Internet connection).
					 If 'false' then the connection closed because the {@link #close} method was invoked. (Boolean)<br>
					 See also {@link #connect} and {@link #close}.<br><br>
					<b>Event parameters</b><br>
					<i>ServerInitiated</i> Boolean value.
		@example 
				<code>
					function serverClosed(ServerInitiated:Boolean) {
						if (ServerInitiated) {
							trace("Connction closed by server");
						} else {
							trace("Connction closed by Flash");
						}
					}
					es.connectionClosed = serverClosed;
				</code>
	*/
	public var connectionClosed:Function;
	
	/*
		Event fired when the user variables on a user in your room have changed.
		@description This event is fired when a uservariable is created or updated that is associated with a user in your room.<br>
					See also {@link #createUserVariable} and {@link #deleteUserVariable}.<br><br>
						<b>Event parameters</b><br>
						<i>user</i> object referencing the user who owns the variable (Object)<br>
						<i>type</i> Either "created" or "updated" (String)<br>
						<i>name</i> Name of the variable created or updated (String)<br>
		@example
				<code>
					es.userVariableUpdated = function(user, type, name) {
						trace("----------------");
						trace("es.userVariableUpdated called");
						trace("user="+user.Name.value);
						trace(type)
						trace("variable name="+name);
						trace("variable value="+user.userVariables[name])
					}
				</code>
	*/
	public var userVariableUpdated:Function;
	
	/*
		Event fired when the rooms in your zone are loaded as a result of the {@link #getRoomsInZone} method.
		@description This event is fired when the list of rooms in a zone is loaded. The list of rooms is passed in as the
					first parameter (see example for accessing the room names off of those room objects). The second parameter 
					is the name of the zone.<br>
					See also {@link #getRoomsInZone}.<br><br>
					<b>Event parameters</b><br>
					<i>roomList</i> List of rooms (Array)<br>
					<i>zone</i> Name of zone (String)
		@example
				<code>
					es.getRoomsInZone("Chat Area");
					es.onRoomsInZoneLoaded = function(room_list, zone_name) {
						//This is fired when the rooms have been loaded
						//tracing zone name
						trace(zone_name);
						for (var i = 0; i < room_list.length;++i) {
							var ob = room_list[i];
							//tracing room name
							trace(ob.Name.value);
						}
					};
				</code>
	*/
	public var onRoomsInZoneLoaded:Function;
	
	/*
		Event fired when the number of logged in users is received as a result of {@link #getLoggedInUserCount}.
		@description 
					<b>Event parameters</b><br>
					<i>count</i> The number of users logged into ElectroServer
		@example
			<code>
				es.loggedInUserCountUpdated = function(count:Number) {
					trace("users connected: "+count);
				}
				es.getLoggedInUserCount();
			</code>
	*/
	public var loggedInUserCountUpdated:Function;
	
	/*
		Event fired when the server time is loaded as a result of {@link #getServerTime}.
		@description 
					<b>Event parameters</b><br>
					<i>time</i> The time in miliseconds since 1970 as seen be ElectroServer
		@example
				<code>
					es.onGetServerTime = function(time:Number) {
						trace("server time: "+time);
					}
					es.getServerTime();
				</code>
	*/
	public var onGetServerTime:Function;
	
	/*
		Event fired when a raw message is received from a plugin
		@description This is used by advanced users. Server-side plug-ins can be used to send messages of any format to a user. 
					If the message is not of the typical ElectroServer XML protocol then this event is fired and the raw message 
					is passed through.<br><br>
					This is mainly used for sending extremely condensed messages for real-time games.<br>
					See also {@link #registerRawHandler}.<br><br>
					<b>Event parameters</b><br>
					<i>message</i> Raw message as a string (String)
		@example 
				<code>
					es.rawMessageReceived = function(message) {
						trace(message);
					}
				</code>
	*/
	public var rawMessageReceived:Function;
	
	/*
		Event fired when a buddy in your buddy list logs in
		@description 
					This is fired as a result of one of two events:<br>
					1) That buddy just logged into the server<br>
					2) That buddy was already logged into the server but you just added him to your buddy list. The server
					fires this event so that you are updated to know that the buddy you just added is logged in.<br><br>
					See also {@link #addBuddy} and {@link #removeBuddy}.<br><br>
					
					<b>Event parameters</b><br>
					<i>username</i> Name of buddy who just logged in (String)
		@example 
				<code>
					es.buddyLoggedIn = function(name) {
						//This is fired when a buddy of yours logs in
						trace(name +" just logged in")
					};
				</code>
	*/
	public var buddyLoggedIn:Function;
	
	/*
		Event fired when a buddy in your buddy list logs out
		@description This is fired as a result of one of two events:<br>
					1) That buddy just logged out of the server<br>
					2) That buddy was already logged out of the server but you just added him to your buddy list.
					The server fires this event so that you are updated to know that the buddy you just added is not online.<br><br>
					<b>Event parameters</b><br>
					<i>username</i> Name of buddy that just logged out (String)
		@example
				<code>
					es.buddyLoggedOut = function(name) {
						//this is called when a buddy of yours logs out
						trace(name+" just logged out");
					};
				</code>
	*/
	public var buddyLoggedOut:Function;
	
	/*
		Event fired when the location of a user has been loaded as a result of {@link #getUserLocation}
		@description This event is fired as a result of {@link #getUserLocation}. It tells you the zone and room that a specific
					user is in. See {@link #getUserLocation} for more information.<br><br>
					<b>Event parameters</b><br>
					<i>username</i> Username (String)<br>
					<i>zone</i> Zone that the user is in (String)<br>
					<i>room</i> Room that the user is in (String)
		@example 
				<code>
					es.getUserLocation("jobem");
					es.userLocationLoaded = function(username, zone, room) {
						trace(username+", "+zone+", "+room);
					};
				</code>
	*/
	public var userLocationLoaded:Function;
	
	/*
		Event fired when the usrs in a specific room have been loaded as a result of {@link #getUsersInRoom}
		@description As a result of getUsersInRoom this event is fired. An array of user objects with a 'name'
					property is passed in. Also, the zone name and room name of the users are passed in.<br><br>
					Note: we are using objects with a 'name' property to leave open the chance to add more useful 
					properties to this in the future, like userVariables. Currently you cann't load userVariables of a 
					user who is in another room.<br><br>
					
					<b>Event parameters</b><br>
					<i>users</i> Array of objects (Array)<br>
					<i>zone</i> Name of zone that room is in (String)<br>
					<i>room</i> Name of room that users are in (String)
		@example
				<code>
					es.getUsersInRoom("Chat Area", "Lobby");
					es.usersInRoomLoaded = function(users, zone, room) {
						//This is fired when the users in a specific room have been loaded
						//array is returned with user objects. They have only 1 property, username
						trace(zone)
						trace(room)
						for (var i = 0; i var ob = users[i];
							trace(ob.username);
						}
					};
				</code>
	*/
	public var usersInRoomLoaded:Function;
	
	/*
		Event fired when a list of zones are loaded as a result of {@link #getAllZones}
		@description This event is fired when the list of all zones on the server is loaded. An array of zone names 
					is passed in. See also {@link #getAllZones}.<br><br>
					<b>Event parameters</b><br>
					<i>zones</i> List of zone names (Array)
		@example  
				<code>
					es.getAllZones();
					es.allZonesLoaded = function(zones) {
						//This is called when all zone names have been loaded
						for (var i = 0; i //tracing zone names
							trace(zones[i]);
						}
					};
				</code>
	*/
	public var allZonesLoaded:Function;
	//@exclude Properties
	public var ignore2;
	
	//A pointer to the objec that represents you
	public var myUser:Object;
	
	//'true' if you are in a game room
	public var inGame:Boolean;
	private static var _instance:ElectroServer;
	private var MajorVersion:Number = 3;
	private var MinorVersion:Number = 2;
	private var SubVersion:Number = 0;
	private var debug:Boolean = false;
	private var showModeratorLabel:Boolean = false;
	private var moderatorLabel:String;
	private var ip:String;
	private var port:Number;
	private var ConnectedToServer:Boolean;
	private var pendingRoom:String;;
	private var joiningRoom:Boolean
	private var pendingZone:String;
	private var zone:Object;
	private var auto_join:Boolean;
	
	//'true' if you are the first on in a game room. (Not used much anymore)
	public var isGameMaster:Boolean;
	private var configXML:String;
	
	//Your username. Can alos be referenced like this: es.getUser().Name.value
	public var username:String;
	
	//Your password
	public var password:String;
	private var buddyList:Array;
	private var nodeNameList:Object = {Users:true, Zones:true, UserVariables:true, RoomVariables:true, Rooms:true, Variables:true, BannedUsers:true, Plugins:true, Moderators:true, Words:true, RootWords:true, Templates:true};
	private var isConnected:Boolean = false;
	/*
		@description This class is a singleton, so the constructor is private. To create a new instance of
		this class use the 'getInstance()' method.
	*/
	private function ElectroServer() {
		buddyList = new Array();
		zone = new Object();
		zone.rooms = new Array();
	}
	private function ValidateVersion(Major:Number, Minor:Number, Sub:Number) {
		var ReturnVal:Boolean = true;
		if (MajorVersion>Major || isNaN(Major)) {
			ReturnVal = false;
		} else if (MinorVersion>Minor && MajorVersion == Major) {
			ReturnVal = false;
		} else if (SubVersion>Sub && MinorVersion == Minor) {
			ReturnVal = false;
		}
		return ReturnVal;
	}
	//
	private function onClose() {
		isConnected = false;
		connectionClosed(true);
	}
	/*
		Removes a buddy from your list
		@description Removes a user from your buddy list
		@param Username to remove
		@example <code>var user:String = "jobe";
			es.removeBuddy(user);</code>
	*/
	public function removeBuddy(name:String) {
		for (var i = 0; i<buddyList.length; ++i) {
			if (buddyList[i].username == name) {
				buddyList.splice(i, 1);
				break;
			}
		}
		var action = "DeleteFriend";
		var parameters = "<FriendName>"+name+"</FriendName>";
		send(action, parameters);
	}
	/*
		Loads a list of users for a specific room in a specific zone
		@description Loads a list of users in a specific room and zone. When 
						loading is comple, the 'usersInRoomLoaded' event is fired
		@param The zone that the room is in
		@param The room you are interested in
		@example <code>
					es.usersInRoomLoaded = function(userlist:Array, roomName:String, zoneName:String) {
						trace("--user list loaded for zone: "+zoneName+", roomName");
						for (var i=0;i<userlist.length;++i) {
							var user:Object = userlist[i];
							trace(user.Name.value);
						}
					}
					es.getUsersInRoom("Chat Area", "Lobby");
				</code>
	*/
	public function getUsersInRoom(zone:String, room:String) {
		var action = "GetUsersInRoom";
		var parameters = "<Zone>"+zone+"</Zone><Room>"+room+"</Room>";
		send(action, parameters);
	}
	/*
		Loads the location of any user on the server
		@description Loads the location of a user from the server. When loaded the 'userLocationLoaded' event is fired.
		@param The name of the user
		@example
			<code>
				es.getUserLocation("jobem");
				//Capture the response
				es.userLocationLoaded = function(username, zone, room) {
					trace(username+", "+zone+", "+room);
				};
			</code>
	*/
	public function getUserLocation(name:String) {
		var action = "GetUserLocation";
		var parameters = "<UserName>"+name+"</UserName>";
		send(action, parameters);
	}
	/*
		Tells ElectroServer which plugin to route raw messages to
		@description This is an advanced feature. You call this method and pass in the name of an ElectroServer plugin. Then,
					if the client needs to send a custom formatted message to the server it can. All messages whose formats are not
					recognnized are forwarded on to the plug-in. Instead of a 500 byte XML message you could send a 20 byte custom string.
					This works well for high performance real-time games. See also {@link #sendRaw}.
		@param	The name of the plugin
		@example	es.registerRawHandler("ShootemUpPlugin");
	*/
	public function registerRawHandler(name:String) {
		var action = "RegisterRawHandler";
		var parameters = "<Name>"+name+"</Name>";
		send(action, parameters);
	}
	/*
		Returns an array of your buddies
		@description This method returns the list of your buddies as an array. Each element in the array is an object that has the following properties:
					username - name of the buddy
					isOnline - Boolean (true for online)
					ob.status - String (offline or online)
					ob.label - Matches the username. For use with list box component
					See also {@link #addBuddy}, {@link #removeBuddy}, {@link #buddyLoggedIn}, {@link #buddyLoggedOut}
		@example <code>
					var myBuddies:Array = es.getBuddyList();
					for (var i=0;i<myBuddies.length;++i) {
						var buddy:Object = myBuddies[i];
						trace(buddy.username);
					}
				</code>
				Alternative usage: Also works with a list box component
				<code>
					myList_lb.dataProvider = es.getBuddyList();
				</code>
	*/
	public function getBuddyList():Array {
		return buddyList;
	}
	/*
		Adds a buddy to your list
		@description ElectroServer supports the idea of "buddies". You can add a buddy to your buddy list. 
					You are told when ever a buddy logs into the server or logs out of the server with the buddyLoggedIn and buddyLoggedOut events.
					The moment you add a buddy to your list you are informed if that buddy is logged in or not.
					See also {@link #removeBuddy}, {@link #buddyLoggedIn}, {@link #buddyLoggedOut}, {@link #getBuddyList}
		@param	Name of user to add
		@example <code>es.addBuddy("jobem");</code>
	*/
	public function addBuddy(name:String) {
		var buddyExists:Boolean = false;
		for (var i = 0; i<buddyList.length; ++i) {
			if (buddyList[i].username == name) {
				buddyExists = true;
				break;
			}
		}
		if (!buddyExists) {
			var ob = new Object();
			ob.isOnline = false;
			ob.username = name;
			ob.status = "offline";
			ob.label = ob.username+" "+ob.status;
			buddyList.push(ob);
			/////////////////////////////
			var action = "AddFriend";
			var parameters = "<FriendName>"+name+"</FriendName>";
			send(action, parameters);
		}
	}
	/*
		Closes the connection with ElectroServer
		@description Closes the connection to ElectroServer. As a result the {@link #connectionClosed} event is fired. See also {@link #connectionClosed}.
		@example <code>es.close();</code>
	*/
	public function close() {
		super.close();
		connectionClosed(false);
	}
	/*
		Loads the time from the server
		@description Loads the server time in miliseconds. This absolute number itself is rarely useful. This is usually useful
					when trying to syncrhonize the client with logic on the server. When the time is loaded the {@link #onGetServerTime} 
					event is fired. See also {@link #onGetServerTime}.
		@example
				<code>
					es.onGetServerTime = function(time:Number) {
						trace("server time: "+time);
					}
					es.getServerTime();
				</code>
	*/
	public function getServerTime() {
		var action:String = "GetServerTime";
		var parameters:String = "";
		send(action, parameters);
	}
	/*
		Loads the number of connect users from the server
		@description Loads the total number of users connected to ElectroServer. When load is complete {@link #loggedInUserCountUpdated} is fired.
					See also {@link #loggedInUserCountUpdated}.
		@example
			<code>
				es.loggedInUserCountUpdated = function(count:Number) {
					trace("users connected: "+count);
				}
				es.getLoggedInUserCount();
			</code>
	*/
	public function getLoggedInUserCount() {
		var action:String = "GetLoggedInUserCount";
		var parameters:String = "";
		send(action, parameters);
	}
	/*
		Sets the moderator label to be appended to the front of a the name of all moderators
		@description All user objects have a 'label' property. If that user is a moderator its label can look different than other users.
					To do that use this method. For instance, a moderator may have a label that looks like this "[mod] Jobe". Over all, 
					this isn't used very often.
		@param A Boolean value. If true then it will show the moderator label.
		@param The string to append to the front of a moderator's label
		@example <code>es.setModeratorLabel(true, "[mod]");
	*/
	public function setModeratorLabel(val:Boolean, str:String) {
		showModeratorLabel = val;
		moderatorLabel = str;
		if (str == undefined) {
			moderatorLabel = " [Moderator]";
		}
	}
	/*
		Returns an instance of the ElectroServer class
		@description The ElectroServer class is a singleton. You don't use it like this: var es = new ElectroServer(). You get the instance
					of ElectroServer like this: var es = ElectroServer.getInstance();
		@example <code> var es:ElectroServer = ElectroServer.getInstance();</code>
	*/
	public static function getInstance():ElectroServer {
		if (_instance == null) {
			_instance = new ElectroServer();
		}
		return _instance;
	}
	/*
		Sets the debug property to true or false. If true it traces all inbound/outbound messages
		@description This is used to trace all inbound and outbound messages used to communicate with ElectroServer. By default it is not enabled.
					It can be very useful when debugging issues. See also {@link #getDebug}.
		@param Boolean value. If true then debugging is enabled.
		@example <code>es.setDebug(true);</code>
	*/
	public function setDebug(val:Boolean) {
		debug = val;
	}
	/*	
		Returns the debug state, true or false
		@description Returns either true or false. You can change the debug state using setDebug(true/false);. See also {@link #setDebug}.
		@example <code>var debugVal:Boolean = es.getDebug();</code>
	*/
	public function getDebug():Boolean {
		return debug;
	}
	/*
		Returns the IP address that ElectroServer is using
		@description Returns the IP address of ElectroServer
		@example <code>var ip:String = es.getIP();</code>
	*/
	public function getIP():String {
		return ip;
	}
	/*
		Returns the port that ElectroServer is using
		@description Returns the port that ElectroServer is using
		@example <code>var port:Number = es.getPort();</code>
	*/
	public function getPort():Number {
		return port;
	}
	/*
		Sets the IP address to use when connecting to ElectroServer
		@description Sets the ip address of ElectroServer. When ElectroServer 3 is started (on a physical
					server somewhere or on your local machine) it binds to a specific IP address. You must 
					specify this IP address and the port in order to connect to ElectroServer.
		@param The ip address to connect to.
		@example 
		<code>
			es.setIP("127.0.0.1");
			es.setPort(9875);
			es.connect();
		</code>
	*/
	public function setIP(tempIP:String) {
		ip = tempIP;
	}
	/*
		Sets the port to use when connecting to ElectroServer
		@description Sets the port that the class will use to connect to ElectroServer. ElectroServer must be listening on this port
		@param	The port to use
		@example
		<code>
			es.setIP("127.0.0.1");
			es.setPort(9875);
			es.connect();
		</code>
	*/
	public function setPort(tempPort:Number) {
		port = tempPort;
	}
	/*
		Deletes a room variable
		@description Deletes the room variable with the name passed in, even if it is a locked room variable.
					For more information on room variables, see {@link #createRoomVariable} and {@link #getRoomVariables}.
		@param The name of the room variable to delete
		@example	<code>es.deleteRoomVariable("welcomeMessage");</code>
	*/
	public function deleteRoomVariable(name:String) {
		var action:String = "DeleteRoomVariable";
		var parameters:String = "<RoomVariable ><Name>"+name+"</Name></RoomVariable>";
		send(action, parameters);
	}
	/*
		Loads the list of rooms in a specific zone
		@description Loads a room list for a specific zone. When loaded 'onRoomsInZoneLoaded' is fired. See also {@link #onRoomsInZoneLoaded}.
		@param The name of the zone
		@example
		<code>
			es.onRoomsInZoneLoaded = function(roomlist:Array, zoneName:String) {
				trace("Room list loaded for " +zoneName);
				for (var i=0;i<roomlist.length;++i) {
					var room:Object = roomlist[i];
					trace(room.Name.value);
				}
			}
			es.getRoomsInZone("Golf Zone");
		</code>
	*/
	public function getRoomsInZone(tmp_zone:String) {
		var action:String = "GetRoomsInZone";
		var parameters:String = "<Zone>"+tmp_zone+"</Zone>";
		send(action, parameters);
	}
	/*
		Changes one of many properties in a room
		@description Change attributes of the room that you are currently in. Acceptable 'details' are:<br>
					<b>description</b> The value is an object (just like in createRoom and createGame). The properties 
					of the object are description and attributes. Attributes is another object that can hold any number of
					custom variables.<br>
					<b>updatable</b> The value is true or false. If true, then the room will receive updates
					about the zone. If false then it will receive no updates accept regarding itself.<br>
					<b>hidden</b> The value is true or false. If true then the room is not seen by any other room.
					If false then the room is seen by other rooms within the zone.<br>
					<b>capacity</b> The value is a number. This changes the max capacity fo the room. If the value 
					is -1 then there is no limit.<br>
					<b>password</b> The value is a string or blank. If blank then the room is switched to having no 
					password. If not blank then the password of the room is changed to the new value.
		@param The name of the attribute to change
		@param The new value for the attribute that is changing
		@example
		Makes a room hidden:
		<code>
			es.changeRoomDetail("hidden", true);
		</code>
		Changes the password of your room:
		<code>
			es.changeRoomDetail("password", "myNewPass");
		</code>
		Changes the description object:
		<code>
			var ob:Object = new Object();
			ob.attributes = new Object();
			ob.description = "A cool room";
			ob.attributes.some_useful_variable = "asdf";
			es.changeRoomDetails("description", ob);
		</code>
	*/
	public function changeRoomDetail(detail:String, value) {
		var action:String = "ChangeRoomDetails";
		var minorAction:String;
		var field:String;
		detail = detail.toLowerCase();
		var pwordXML:String = "";
		if (detail == "description") {
			minorAction = "ChangeDescription";
			field = "Description";
			value.attributes.isGameRoom = true;
			var wddxOb = new Wddx();
			var descXML = "<![CDATA["+wddxOb.serialize(value)+"]]>";
		} else if (detail == "updatable") {
			minorAction = "ChangeUpdatable";
			field = "Updatable";
			var descXML:String = value;
		} else if (detail == "hidden") {
			minorAction = "ChangeVisibility";
			field = "Hidden";
			var descXML:String = value;
		} else if (detail == "capacity") {
			minorAction = "ChangeCapacity";
			field = "Capacity";
			if (value == undefined || value == 0) {
				value = -1;
			}
			var descXML:String = value;
		} else if (detail == "password") {
			minorAction = "ChangePasswordProtected";
			field = "Password";
			var descXML:String = value;
			var protected:Boolean;
			if (value == undefined) {
				value = "";
			}
			if (value.length>=1) {
				protected = true;
			} else {
				protected = false;
			}
			pwordXML = "<IsProtected>"+protected+"</IsProtected>";
		}
		var parameters:String = "<MinorAction>"+minorAction+"</MinorAction>"+pwordXML+"<"+field+">"+descXML+"</"+field+">";
		send(action, parameters);
	}
	/*
		Kicks a user from the server
		@description Kicks a user from the server. This is only available to moderator level users. A moderator is specified 
					by a specific username/password combination found in the configuration.xml file.
		@param Name of the user to be kicked
		@param Reason for kicking
		@example <code>es.kick("jobe", "for being too cool");</code>
	*/
	public function kick(name:String, reason:String) {
		if (reason == undefined) {
			reason = "";
		}
		var action:String = "ModeratorCommand";
		var parameters:String = "<MinorAction>Kick</MinorAction><UserName>"+name+"</UserName><Reason>"+reason+"</Reason>";
		send(action, parameters);
	}
	/*
		Kicks and bans a user from the server
		@description Kicks a user from the server and bans them from logging in again. The 'time' parameter is the length
		of time in minutes that a user should be banned from the server. If "-1" then the user is banned until the the
		server is restarted.
		@param Name of user to be banned
		@param Reason for banning
		@param The amount of time that the user should be banned in minutes
		@example
		The following bans someone for 20 minutes.
		<code>es.ban("jobem", "You have repeatedly been mean.", 20);</code>
	*/
	public function ban(name:String, reason:String, expires) {
		if (reason == undefined) {
			reason = "";
		}
		if (expires == undefined) {
			expires = "-1";
		}
		var action:String = "ModeratorCommand";
		var parameters:String = "<MinorAction>Ban</MinorAction><UserName>"+name+"</UserName><Reason>"+reason+"</Reason><Expires>"+expires+"</Expires>";
		send(action, parameters);
	}
	/*
		Creates a user variable
		@description Creates a variable stored on the server associated with the user that created it. As the user 
					moves from room to room the variable comes with him. When the user enters a room or when any 
					uservariable is created, removed, or modified, all users in that room are notified of the change.
					See also {@link #deleteUserVariable}, {@link #userVariableUpdated}. Note: There is an example file installed
					with ElectroServer called UserVariablesTest.fla that is very helpful.
		@param Name of the user variable
		@param Value of the variable
		@example 
		To create a user variable on yourself:
		<code>es.createUserVariable("myIcon", "happy face");</code>
		To trace everyone's user variables:
		<code>
		var userlist:Array = es.getUserList();
		for (var i=0;i<userlist.length;++i) {
			var user:Object = userlist[i];
			var userVariables:Object = user.userVariables;
			trace("User variables for "+user.Name.value);
			for (var name in userVariables) {
				trace(" variable name: "+name);
				trace(" variable value: "+userVariables[name]);
			}
		}
		</code>
	*/
	public function createUserVariable(name:String, value:String) {
		var action:String = "UpdateUserVariable";
		var parameters:String = "<MinorAction>Create</MinorAction><Name>"+name+"</Name><Data>"+value+"</Data>";
		send(action, parameters);
	}
	/*
		Loads a list of all the zones currently established
		@description Loads the list of zones on the server. When loaded 'allZonesLoaded' event is fired. See also {@link #allZonesLoaded}.
		@example
		<code>
			es.allZonesLoaded = function(zones:Array) {
				trace("--All Zones--");
				for (var i=0;i<zones.length;++i) {
					trace(zones[i]);
				}
			}
			es.getAllZones();
		</code>
	*/
	public function getAllZones() {
		send("GetAllZones", "");
	}
	/*
		Updates a user variable
		@description This updates an existing user variable. It works just like creating a user variable. See also {@link #createUserVariable}, {@link #deleteUserVariable}.
		@param The name of the variable to update
		@param The new value of the variable
		@example <code>es.updateUserVariable("myIcon", "sadFace");</code>
	*/
	public function updateUserVariable(name:String, value:String) {
		var action:String = "UpdateUserVariable";
		var parameters:String = "<MinorAction>Update</MinorAction><Name>"+name+"</Name><Data>"+value+"</Data>";
		send(action, parameters);
	}
	/*
		Deletes a user variable
		@description Deletes an existing user variable. See also {@link #createUserVariable}, {@link #updateUserVariable}.
		@param	Name of variable to be deleted
		@example <code>es.deleteUserVariable("myIcon");
	*/
	public function deleteUserVariable(name:String) {
		var action:String = "UpdateUserVariable";
		var parameters:String = "<MinorAction>Delete</MinorAction><Name>"+name+"</Name>";
		send(action, parameters);
	}
	/*
		Creates a room variable
		@description Creates or updates a variable in the room on the chat server. The variable can
		be flagged as locked and/or persistent.<br>
		<b>Deeper explanation</b><br>
		ElectroServer supports room variables. This means that a Flash client can create a variable at the room level 
		that is actually stored in ElectroServer. While ElectroServer is the 'master' of all of these room variables, each Flash 
		client knows about all of the room variables as well. When ever a room variable is created, modified, or deleted, every 
		user in that room is notified and the ElectroServer object updates this information internally. <br>
		When room variables are created, modified, or deleted an event in the ElectroServer object called {@link #roomVariablesUpdated} 
		is fired. See that event for more information. The room variables can be accessed at any time by using the {@link #getRoomVariables}
		method.<br>
		Room variables can be created and then modified. A room variable can be created and locked so that it cannot be modified 
		any more (but can still be deleted). And a room variable can be flagged as persistent, which means that it is not automatically
		deleted when the user that created it leaves the room.<br>
		<b>Why room variables are useful</b><br>
		With room variables you have a centralized location to store data. If, for instance, you wanted to
		create an avatar chat where chracters could walk around and chat, then room variables can help in the following way:<br>
		For each user in the room you can create a room variable that stores his position and the type of character
		that he is using. Then when someone new joins the room their Flash client just uses the room variables to determine
		where to place the avatar characters. Everytime someone moves their avatar character then they send a room variable
		update to update its postion on the server.<br>
		Without room variables the way that the above would be accomplished would be by sending information separately
		to everyone in the room round-robin style. This is not good practice, causes synchronization issues, and uses more bandwidth.<br>
		Here are two good examples of persistent room variables (meaning that they do not delete when the user that created them leaves
		the room):<br>
		1.) A Flash white board. You can use persistent room variables to store the drawings. 
		Then as people leave and new people come into the white board they can see everything.<br>
		2.) For a multiplayer game you can build something in the world or leave something behind 
		(like scorch marks form an explosion). When you leave the variable remains and so everyone can see your contribution.<br>
		<b>Here is a good example of a locked room variable:</b><br>
		Let's say you wanted to create a scavenger hunt game. This game has several people in it and there are say, 
		10 objects to collect. You can collect one by moving your character to the object and pushing a special key. 
		Well, what if two people reach the same object at the same time? How do you know who actually collected it? 
		There is no way to do this without server-side help, like locked room variables.<br>
		If your username is "jobem" and you collect a "gold cup" then you can set a locked user variable into the room called:<br>
		goldCup = "jobem"<br>
		When everyone receives this room variable update they can flag that object as collected. If someone tries to 
		collect the object in that 100 or so miliseconds of Internet latency then they will still fail because they will
		not be able to create the room variable as it is already locked!<br>
		To 'clean up' the game to start over you would then have to delete each of those room variables.
		@param An object containing many optional properties. Here are those properties:<br>
			<b>name</b> The name of the room variable to be created or updated. (String)<br>
			<b>data</b> The value of the room variable to be created or updated. (String)<br>
			<b>locked</b> Optional property. If true then the variable can not be modified once created in the room, 
			however it can still be deleted. The default value is false. (Boolean)<br>
			<b>persistent</b> Optional property. If true then the variable remains in the room after the user that 
			created it leaves the room. If false then the variable is deleted when the user who created it leaves the room. 
			The default value is false. 
		@example
		The following code creates a room variable on the server in your current room. The variable is not locked and
		will be removed when you leave the room:
		<code>
			var ob = new Object();
			ob.name = "NumberOfBadGuys";
			ob.data = "30";
			es.createRoomVariable(ob);
		</code>
		The following code creates a room variable on the server in your current room. This variable is locked, meaning that
		it cannot be modifed (only deleted). It is also persistent, which means it will not die if you leave the room:
		<code>
			var ob:Object = new Object();
			ob.name = "secretDoorUnlocked";
			ob.data = "true";//this is a string because all room variables are strings
			ob.locked = true;
			ob.persistent = true;
			es.createRoomVariable(ob);
		</code>


	*/
	public function createRoomVariable(ob:Object) {
		var name:String = ob.name;
		var data = ob.data;
		var persistent:Boolean = ob.persistent;
		var locked:Boolean = ob.locked;
		if (persistent == undefined) {
			persistent = false;
		}
		if (locked == undefined) {
			locked = false;
		}
		if (zone.myRoom.roomVariables[name] == undefined) {
			var action:String = "CreateRoomVariable";
			var parameters:String = "<RoomVariable Persistent=\""+persistent+"\" Locked=\""+locked+"\" ><Name>"+name+"</Name><Data><![CDATA["+data+"]]></Data></RoomVariable>";
		} else {
			var action:String = "UpdateRoomVariable";
			var parameters:String = "<RoomVariable Locked=\""+locked+"\" ><Name>"+name+"</Name><Data><![CDATA["+data+"]]></Data></RoomVariable>";
		}
		send(action, parameters);
	}
	/*
		Returns a reference to the zone that you're in
		@description Returns an object representing your current zone. This object has the following properties:<br>
					numUsers - This is number of people in your zone.<br>
					name - This is the name of the zone that you are in.
		@example	<code>
					var myZone:Object = es.getZone();
					trace(myZone.name);
					trace(myZone.numUsers);
				</code>
		@returns An object representing the zone that you are in.
	*/
	public function getZone() {
		return zone;
	}
	/*
		Sends an object to the room or to an array of specific users
		@description Sends an object to the room or to an array of specific users. Using the sendMove method you
					actually send an ActionScript object to the room or to specific users. The object is serialized
					using the WDDX protocol, sent to the user, deserialized, and used.<br>
					When the user receives the object the {@link #moveReceived} event is fired.<br>
					Note: The WDDX serialization ActionScript classes are used courtesy of Branden Hall.
		@param Should be the string "all" or an array of user names. (String or Array)
		@param An ActionScript object to send to the other users (Object)
		@example Here is a simple example of how to send a chess move to another user. 
				<code>
					var ob:Object = new Object();
					ob.action = "move_piece";
					ob.piece = "white_queen";
					ob.y_tile = 7;
					ob.x_tile = 6;
					es.sendMove("all", ob);
				</code>
				Here is an example of how you could send an array to everyone in your room.
				<code>
					var ob:Object = new Object();
					ob.action = "ListOfFriends";
					ob.friends = new Array ("Jobe", "Mike", "Robert", "Kelly");
					es.sendMove("all", ob);
				</code>
				What follows is an example of a private game move that is send to only 2 other people. Let's say they are on
				your team in the game. You are giving them some ammo.
				<code>
					var ob:Object = new Object();
					ob.action = "GiveAmmo";
					ob.ammoType = "Uzi";
					es.sendMove(["mike", "robert"], ob);
				</code>
	*/
	public function sendMove(who, ob:Object) {
		var wddxOb:Wddx = new Wddx();
		var moveXML:XML = wddxOb.serialize(ob);
		if (who.toLowerCase() == "all") {
			var action:String = "SendPublicMessage";
			var parameters:String = "<Message><![CDATA["+moveXML+"]]></Message><Variables><Variable><Name>Action</Name><Data>Move</Data></Variable><Variable><Name>MoveType</Name><Data>Public</Data></Variable></Variables>";
		} else {
			var action:String = "SendPrivateMessage";
			var users:Array = who;
			var usersXML:String = "<Users>";
			for (var i = 0; i<users.length; ++i) {
				var user:String = users[i];
				usersXML += "<User>"+user+"</User>";
			}
			usersXML += "</Users>";
			var parameters:String = usersXML+"<Message><![CDATA["+moveXML+"]]></Message><Variables><Variable><Name>Action</Name><Data>Move</Data></Variable><Variable><Name>MoveType</Name><Data>Public</Data></Variable></Variables>";
		}
		send(action, parameters);
	}
	/*
		Sends a message to a plugin
		@description Sends a request to a custom-written plugin in ElectroServer. You must specify the name
					of the plugin and the method to be executed. The plugin accepts this information and returns
					a result if necessary.<br>
					ElectroServer 3.0 has the ability to accept plugins. Plugins can be written in Java or ActionScript.
					With ElectroServer plugins you can easily extend the functionality of ElectroServer and create applications
					such as multiplayer real-time games! Plugins are super powerful and super cool. Check out <a href="http://www.electro-server.com" target="_parent">http://www.electro-server.com</a>
					for more information on them.
		@param Name of the plugin that you are trying to communicate with. This is specified in the configuration.xml file.
		@param This is a string that the plugin will use to determine what to do with your request.
		@param This is an optional parameter, an object of name value pairs. All values should be strings.
		@example
				The following could work with a plugin written to store white board data. The white board could then
				inform all new users to the room of the current state of the board. This code could tell the plugin to 
				draw a line. It is then up to the plugin to send this information to the other people in the room.
				<code>
					var parameters:Object = new Object();
					parameters.startx = 200;
					parameters.endx = 300;
					parameters.starty = 37;
					parameters.endy = 110;
					parameters.color = "blue";
					es.pluginRequest("WhiteBoardScriptPlugin", "DrawLine", parameters);
				</code>
	*/
	public function pluginRequest(plugin:String, method:String, parameters:Object) {
		var action:String = "ExecutePlugin";
		var variableXML:String = "<Variables />";
		if (parameters != undefined) {
			variableXML = "<Variables>";
			for (var i in parameters) {
				variableXML += "<Variable><Name>"+i+"</Name><Value>"+parameters[i]+"</Value></Variable>";
			}
			variableXML += "</Variables>";
		}
		var params:String = "<Plugin>"+plugin+"</Plugin><Method>"+method+"</Method>"+variableXML;
		send(action, params);
	}
	/*
		Returns an object full of variables
		@description Returns an object whose properties are ElectroServer room variables. Each property of this object
					is a variable stored at the room level actually on the server. These properties can be accessed either directly
					(like object.myVariable) or by using a 'for...in' loop. See the example.<br>
					See also {@link #createRoomVariable}, {@link #deleteRoomVariable}
		@example 
				To simply show all of the room variables for a room in the output window, use the following code:
				<code>
					function showRoomVariables() {
						var roomVariables:Object = es.getRoomVariables();
						for (var i in roomVariables) {
							trace(i+"="+roomVariables[i]);
						}
					}
				</code>
				Or you can access a room variable directly like the following:
				<code>
					var musicLoop:String = es.getRoomVariables().musicLoop;
				</code>

		@returns An object. Each property in the object is a variable for that room.
	*/
	public function getRoomVariables() {
		return zone.myRoom.roomVariables;
	}
	/*
		Returns an array of rooms
		@description Returns the list of rooms in your zone as an array. Each element in the array in an 
					object describing a room. From each element you gain information about a room such as its name,
					description, number of users, maximum number of users allowed, and if the room is password protected.<br>
					Note: A zone is a collection of rooms much like a room is a collection of users.<br>
					<b>Properties of each room object returned in the array</b><br>
					Each element in the room list array is an object describing a room. Here are the properties of a room object:<br>
					<b>Name.value</b> The name of the room<br>
					<b>Description.value</b> The room description if there is one.<br>
					<b>attributes</b> (an object containing the following room properties):<br>
					<i>attributes.users</i> The number of users in the room<br>
					<i>attributes.MaxCapacity</i> The maximum number of users allowed in this room. '-1' means there is no limit.<br>
					<i>attributes.IsPasswordProtected</i> A boolean value specifying if the room requires a password to enter.<br>
					<i>attributes.Users</i> The number of people in that room.<br>
					See also {@link #createRoomVariable}, {@link #deleteRoomVariable}.
		@example 
				The code that follows is a simple way to just display the rooms in your current zone in an HTML enabled
				text filed. Note: this does not make use of the the other properties of the room objects.
				<code>
					function showRooms() {
						var roomListBoxStr:String = "";
						var roomlist:Array = es.getRoomList();
						for (var i = 0; i<roomlist.length; ++i) {
							var room:Object = roomlist[i];
							roomListBoxStr += room.name.value+"<br>";
						}
						roomListBox.htmlText = roomListBoxStr;
					}
				</code>
		@returns An array. Each element in the array is an object.
	*/
	public function getRoomList() {
		return zone.rooms;
	}
	/*
		Returns a reference to the room you're in
		@description Returns the room object that describes your room. This object contains information about your
					room such as it's name, number of users in it, if it is password protected, its maximum capacity,
					and all of the room variables. For more information about the properties on the room object see {@link #getRoomList}.
		@example <code>var myRoomName:String = es.getRoom().Name.value</code>
		@returns Object that describes the room that you are in.
	*/
	public function getRoom() {
		return zone.myRoom;
	}
	/*
		Returns an array of users
		@description Returns an array of objects each of which describes a user in your room. The properties of
					each object are 'Name.value' which is a string, and 'moderator' which is a boolean value. If 'moderator'
					is true then that user is a moderator. There is also a property called 'userVariables'.<br>
					User variables are variables saved on the server and associated with a specific user. This is useful
					for carrying information about a user around from room to room. See {@link #createUserVariable} for more
					information on user variables.<br>
					See also {@link #getUser}.
		@example This gets the user list, loops through it, and traces out the names of the users.
				<code>
				var list:Array = es.getUserList();
				for (var i=0;i<list.length;++i) {
					var user:Object = list[i];
					trace(user.Name.value);
				}
				</code>
		@returns An array of objects. Each object describes a user in your room.
	*/
	public function getUserList() {
		return zone.users;
	}
	/*
		Creates a room with specific flags
		@description Creates a room for a game. This method accepts an object with exactly the same properties
					as the {@link #createRoom} method does. Please see that method for full details.<br>
					The flag that shows that this room is a game room is found in the room
					object as: roomOb.description.attributes.isGameRoom, and has a value of 'true'.<br><br>
					Note: The createGameRoom method differs from the creatRoom method only because it flags the
					room as being a game room. When the room is created, modified, or removed, the same events are
					fired as with a regular room. By flagging a room as a game room it is easier to show a difference
					in the room list.<br><br>
					<b>Note:</b> This method is not used very frequently. Developers, in general, prefer to just use {@link #createRoom}.
		@param Object describing room to be created
		@example <code>
					var roomOb:Object = new Object();
					roomOb.zone = "Game Zone";
					roomOb.roomName = "DiePleez";
					es.createGameRoom(roomOb);
				</code>
	*/
	public function createGameRoom(roomOb:Object) {
		if (roomOb.attributes == undefined) {
			roomOb.attributes = new Object();
		}
		roomOb.attributes.isGameRoom = true;
		if (roomOb.updatable != true) {
			roomOb.updatable = false;
		}
		roomOb.numbered = true;
		createRoom(roomOb);
		inGame = true;
	}
	/*
		Joins you to a game room
		@description A game exists in a room. To join that game you must join that room. This method handles that for you.<br>
					If you pass in a playerType of "player" then you are joining that game to become a player. 
					If you pass in a playerType of "spectator" then you are just there to watch.
		@param The name of the room
		@param The password for the room, if any.
		@param Either "player" or "spectator" depending on why you're joining the game.
		@param The name of the zone if it differs from the one you're currently in.
		@example <code>es.joinGame("Mini Golf 173", "", "player");</code>
	*/
	public function joinGame(room:String, password:String, type:String, zone:String) {
		type = type.toLowerCase();
		if (type == "player" || type == undefined) {
			var numbered:Boolean = true;
		} else if (type == "spectator") {
			var numbered:Boolean = false;
		}
		joinRoom(room, password, zone, numbered);
		inGame = true;
	}
	/*
		Special login for admin users
		@description This method is exactly the same as the {@link #login} method except that it informs the server
					that this user is trying to login as an administrator. There is only 1 administrator per 
					instance of ElectroServer. The administrator's username and password are set in the configuration.xml file.<br>
					Administrators, once logged in, have a greater level of access than do users or moderators.
		@param Username of admin
		@param Password of admin
		@example <code>es.adminLogin("system", "electroserver");</code>
	*/
	public function adminLogin(tempUsername:String, tempPassword:String) {
		myUser.username = tempUsername;
		if (tempPassword == undefined) {
			tempPassword = "";
		}
		myUser.password = tempPassword;
		username = myUser.username;
		password = myUser.password;
		var action:String = "AdminLogin";
		var parameters:String = "<Name>"+myUser.username+"</Name><Password>"+myUser.password+"</Password>";
		send(action, parameters);
	}
	/*	
		Loads configuration file if logged in as an admin
		@description Loads the configuration file for the server (only a logged-in admin can do this). Once the configuration
					file is loaded the {@link #configurationLoaded} event is fired.<br>
					The configuration file contains all of the configurable information for ElectroServer 3. It is used
					by admins to configure the server. They can modify many things such as the language filter, plug-ins,
					and banned users.<br><br>
					Admin functionality is not fully documented as it is expected that you will use the provided admin client.
		@example 
				<code>
					es.configurationLoaded = function(ob:Object) {
						//config loaded
					}
					es.loadConfiguration();
				</code>
	*/
	public function loadConfiguration() {
		var action:String = "LoadConfiguration";
		var parameters:String = "";
		send(action, parameters);
	}
	private function parseConfig(config:String) {
		XML.prototype.ignoreWhite = true;
		var temp:XML = new XML(config);
		var params:Object = new Object();
		parseParameters(temp.firstChild.childNodes, params);
		configurationLoaded(params);
	}
	/*
		Sends a public or private message to the room or to an array of specific users
		@description Send a message to the room or an array of specific users. With this method a message can be
						sent to a room (public message) or to a list of users (private message). You can send an unlimited
						list of variables along with any message (see example).<br><br>
						When a message is sent it goes to ElectroServer where it is then broadcast to the appropriate people
						(the room or private users). When the message is received by these users the {@link #messageReceived} event is
						fired. See the {@link #messageReceived} event for more information.<br><br>
						<b>Note:</b> The variables parameter to this method is optional.<br>
						<b>Note:</b> Private messages can be sent to a user anywhere on the server. Each user has a unique
						username (you can't have two "john" users at the same time). So when you send a private
						message to "john" it will find him in any room in any zone.
		@param To send to the room this should be "public" or "all". To send to an array of users it should be "private".
		@param The string message to send.
		@param The array of users to send to. Each element should be a username.
		@param An object of name value pairs. Each value should be a string.
		@example 
				Here is the most simple example of sending a message to a room:
				<code>
					var message:String = "Hello world!";
					es.sendMessage("public", message);
				</code>
				Here is a simple of example of how to send a message to a room with variables:
				<code>
					var message:String = "Hello world!";
					var variables:Object = new Object();
					variables.textColor = "yellow";
					variables.soundToPlay = "boink";
					es.sendMessage("public", message, variables)
				</code>
				Here is a simple example of how to send a private message to three people with variables:
				<code>
					var message:String = "Hello world!";
					var variables:Object = new Object();
					variables.textColor = "yellow";
					variables.soundToPlay = "boink";
					var users:Array = new Array();
					users.push("Kelly");
					users.push("Robert");
					users.push("Mike");
					es.sendMessage("private", message, users, variables);
				</code>

	*/
	public function sendMessage(type:String, message:String, users, variables:Object) {
		var type:String = type.toLowerCase();
		if (type == "public" || type == "all") {
			//'users' is actually the variables for public messages
			sendPublicMessage(message, users);
		} else if (type == "private") {
			sendPrivateMessage(message, users, variables);
		}
	}
	/*
		Joins you to a room
		@description Joins you to a room in a specified zone. You can only join a room that has already been created.
					 If a room has not yet been created then you must first create it using the
					createRoom method, and then ElectroServer will automatically join you to that room. See also {@link #roomJoined}.<br><br>
					The password parameter can be left blank or can be "" if no password is needed. The 'zone' parameter can
					be left blank if you are already in a zone and the room that you are joining is in the same zone. However,
					if you have just logged in to the server and have not yet joined a room, then the zone must be specified.<br><br>
					<b>More detail for beginners</b>
					A room is a collection of users. In a room you can see other users in your room. When in a room you cannot
					see users in any other rooms (you only know that they other rooms exist and you know how many people are in them).
					In a room you can send messages which are received by other people in that room.<br><br>
					Much like a room is a collection of users, a zone is a collection of rooms. In a zone you can see all of the rooms
					in that zone, but you have no information if any other zones exist or any information about other zones.
		@param Name of the room to join
		@param Password for the room, if any (optional)
		@param The name of the zone that the room is in. Only needed if it is in a different zone then you are currently in. (Optional)
		@param Either 'true' or 'false'. If 'true', then you are given a number in the room. This can be useful in some games. (Optional)
		@example <code>es.joinRoom("Lobby", "", "Chat Area");</code>
				or,
				<code>
				es.joinRoom("SecretRoom", "secret_password", "Chat Area");
				</code>
				or if you are already in a zone and want to join another room in that zone with no password, then it can be condensed to this:
				<code>
				es.joinRoom("Game talk");
				</code>
	*/
	public function joinRoom(room:String, password:String, zoneName:String, numbered:Boolean) {
		var action = "JoinRoom";
		if (numbered == undefined) {
			numbered = true;
		}
		if (password == undefined) {
			password = "";
		}
		if (zoneName == undefined) {
			zoneName = getZone().name;
			if (zoneName == undefined) {
				zoneName = "Chat Area";
			}
		}
		pendingZone = zoneName;
		pendingRoom = room;
		inGame = false;
		var parameters:String = "<Zone><Name>"+zoneName+"</Name><Room><Name>"+room+"</Name><Password>"+password+"</Password><Numbered>"+numbered+"</Numbered></Room></Zone>";
		send(action, parameters);
	}
	/*
		Creates a room
		@description Creates a room in a specific zone and then joins you to that room. See below for description
					of each of the configurable properties of a room.<br><br>
					Note: If you attempt to create a room that already exists then internally the ElectroServer class will
					receive an error and you will automatically be joined to that room (since it exists). If that is not behavior
					that you want in your application, then set the 2nd parameter in createRoom to false.<br><br>
					
					<b>Properties of the room object</b><br><br>
					The roomOb object can contain many properties to customize the room. Some of them are required while others are not.<br><br>
					<i>password</i> If blank the room is not password protected. If it contains anything then the room is password protected.<br><br>
					<i>userVariablesEnabled</i> If blank or 'false' then user variables are not enabled for this room (increases performance).
					If 'true', then user variables are enabled for the room. See createUserVariable, login, or userVariableUpdated for
					more information.<br><br>
					<i>hidden</i> If blank or 'false' then the room is not hidden. If 'true', then the room is hidden and cannot be seen
					by the other rooms in your zone. This is often used for game rooms.<br><br>
					<i>zone</i> This is the zone in which you want to create a room. If the zone does not yet exist then it will be
					created. Required.<br><br>
					<i>roomName</i> The name of the room to be created. Required.<br><br>
					<i>numbered</i> If blank or 'false' the room does not number its users. If 'true' then it does. See the 
					{@link #usersRenumbered} event.<br><br>
					<i>capacity</i> If blank or -1 then the room has no maximum size. If the number is greater than 0 then the
					room created can only grow to a certain user size.<br><br>
					<i>description</i> Can be left blank or be a string describing the room.<br><br>
					<i>updatable</i> If blank or 'true' then the room is updatable. If 'false' then it is not. A room that is
					updatable receives zone updates and roomList updates. If it is not updatable then it only receives updates
					regarding itself (userListUpdates, etc). This increases performance for that room, recommended for games.<br><br>
					<i>FloodingFilterEnabled</i> If blank it is set to false, which means it's not enabled. When enabled the room checks
					to see if users send messages too quickly to the server. This is called flooding. If flooding is detected then the 
					user is kicked. Flooding properties can be configured in the configuration.xml file.<br><br>
					<i>userListEnabled</i> The default is true. When enabled, the users in a room are informed of the list of users
					in that the room. Also the room receives {@link #userListUpdated} events when users join or leave. If set to 
					false, then ElectroServer sends no information or updates about the users in a room. This can be useful to set to
					false in rooms that have many users to cut down on message traffic.<br><br>
					<i>roomVariables</i> Can be left blank if you don't want to create a room with room variables. This is an array
					of objects, each of which describes a room variable. The properties of a room variable are 'name'
					(name of variable), 'data' (value of variable), 'persistent' (optional Boolean), 'locked' (optional Boolean).
					Room variables can be created using the {@link #createRoomVariable} method. See that method for more details.<br><br>
					<i>attributes</i> Optional property. This is an object that can contain variables or arrays. It is attached
					to the room much like the room description is. This means that it is accessible by people in other rooms.<br><br>
					<i>plugins</i> Optional property. ElectroServer 3 supports the ability to use server-side plugins. You can access
					custom-written ActionScript or Java plugins from Flash on the server. But in order to access them they must be 
					instantiated when creating a room. This property is an array of objects. Each object represents a plugin that you
					want to instantiate for the room. The 'name' variable on that object is the name of the plugin. The object can 
					contain another object called 'variables' if you want to pass variables into the plugin. Using plugins is an 
					advanced feature that requires a moderate level of understanding of ElectroServer and the ElectroServer object.<br><br>
		@param An object representing the room to be created
		@param Boolean value, default is true. If true and the room that you are creating already exists, then you'll be joined to that room.
		@example 
				Here is the most basic way to create a room:
				<code>
					var roomOb:Object = new Object();
					roomOb.zone = "Chat Area";
					roomOb.roomName = "Lobby";
					es.createRoom(roomOb);
				</code>
				Here is a long-winded example of creating a room with all properties used except plug-ins:
				<code>
					var roomOb:Object = new Object();
					roomOb.zone = "Chat Area";
					roomOb.roomName = "Lobby";
					roomOb.hidden = false;
					roomOb.numbered = false;
					roomOb.userVariablesEnabled = false;
					roomOb.description = "You can chat all day long in this room!";
					roomOb.password = "";
					roomOb.capacity = -1;
					roomOb.updatable = true;
					//room vars
					roomOb.roomVariables = new Array();
					var ob:Object = new Object();
					ob.name = "BackgroundMusic";
					ob.data = "AmbientSounds";
					roomOb.roomVariables.push(ob);
					//end room vars
					es.createRoom(roomOb);
				</code>
				Here is an example creating a room with two plugins:
				<code>
					var roomOb:Object = new Object();
					roomOb.zone = "Chat Area";
					roomOb.roomName = "Lobby";
					roomOb.plugins = new Array();
					roomOb.plugins.push({name:"ShootEmUpPlugin"});
					roomOb.plugins.push({name:"ScoreSaverPlugin"});
					es.createRoom(roomOb);
				</code>
	*/
	public function createRoom(roomOb:Object, auto:Boolean) {
		//roomOb properties: zone, roomName, hidden, numbered, UserVariablesEnabled, Description
		//Password, Capacity, roomVariables
		if (auto == undefined || auto == "true") {
			auto = true;
		} else {
			auto = false;
		}
		auto_join = auto;
		var action:String = "CreateRoom";
		var password:String = roomOb.password;
		var UserVariablesEnabled:Boolean = roomOb.userVariablesEnabled;
		var hidden:Boolean = roomOb.hidden;
		var zoneName:String = roomOb.zone;
		var roomName:String = roomOb.roomName;
		var numbered:Boolean = roomOb.numbered;
		var capacity:Number = roomOb.capacity;
		var description:String = roomOb.description;
		var roomVariables:Array = roomOb.roomVariables;
		var updatable:Boolean = roomOb.updatable;
		var plugins:Array = roomOb.plugins;
		var FloodingFilterEnabled:Boolean = roomOb.FloodingFilterEnabled;
		var userListEnabled:Boolean = roomOb.userListEnabled;
		var descOb:Object = new Object();
		if (userListEnabled == undefined) {
			userListEnabled = true;
		}
		if (description == undefined) {
			description = roomOb.Description;
			if (description == undefined) {
				description = "";
			}
		}
		if (FloodingFilterEnabled == undefined) {
			FloodingFilterEnabled = false;
		}
		descOb.description = description;
		if (roomOb.attributes != undefined) {
			descOb.attributes = roomOb.attributes;
		}
		var wddxOb:Wddx = new Wddx();
		var descXML:XML = wddxOb.serialize(descOb);
		if (zoneName == undefined) {
			zoneName = getZone().name;
			if (zoneName == undefined) {
				zoneName = "Chat Area";
			}
		}
		if (updatable == undefined) {
			updatable = true;
		}
		if (hidden == undefined) {
			hidden = false;
		}
		if (capacity == undefined) {
			capacity = -1;
		}
		if (numbered == undefined) {
			numbered = false;
		}
		if (password == undefined) {
			password = "";
		}
		if (UserVariablesEnabled == undefined) {
			UserVariablesEnabled = false;
		}
		if (roomVariables == undefined) {
			var roomVariableXML:String = "<RoomVariables/>";
		} else {
			var roomVariableXML:String = "<RoomVariables>";
			for (var i = 0; i<roomVariables.length; ++i) {
				var ob:Object = roomVariables[i];
				var persistent:Boolean = ob.persistent;
				var locked:Boolean = ob.locked;
				var name:String = ob.name;
				var data = ob.data;
				if (persistent == undefined) {
					persistent = false;
				}
				if (locked == undefined) {
					locked = false;
				}
				var str:String = "<RoomVariable Persistent=\""+persistent+"\" Locked=\""+locked+"\">";
				str += "<Name>"+name+"</Name>";
				str += "<Data><![CDATA["+data+"]]></Data>";
				str += "</RoomVariable>";
				roomVariableXML += str;
			}
			roomVariableXML += "</RoomVariables>";
		}
		if (plugins == undefined) {
			var pluginXML:String = "<Plugins />";
		} else {
			var pluginXML:String = "<Plugins>";
			for (var i = 0; i<plugins.length; ++i) {
				var plugin:Object = plugins[i];
				var name:String = plugin.name;
				var pluginVariables:Object = plugin.variables;
				if (pluginVariables == undefined) {
					var pluginVariablesXML:String = "<Variables />";
				} else {
					var pluginVariablesXML:String = "<Variables>";
					for (var j in pluginVariables) {
						var pluginVarValue = pluginVariables[j];
						pluginVariablesXML += "<Variable><Name>"+j+"</Name><Value>"+pluginVarValue+"</Value></Variable>";
					}
					pluginVariablesXML += "</Variables>";
				}
				pluginXML += "<Plugin><Name>"+name+"</Name>"+pluginVariablesXML+"</Plugin>";
			}
			pluginXML += "</Plugins>";
		}
		var parameters:String = "<Zone><Name>"+zoneName+"</Name><Room UserListEnabled=\""+userListEnabled+"\" Updatable=\""+updatable+"\" Hidden=\""+hidden+"\" FloodingFilterEnabled=\""+FloodingFilterEnabled+"\" Numbered=\""+numbered+"\" UserVariablesEnabled=\""+UserVariablesEnabled+"\"><Name>"+roomName+"</Name><Password>"+password+"</Password><Description><![CDATA["+descXML+"]]></Description><Capacity>"+capacity+"</Capacity>"+roomVariableXML+pluginXML+"</Room></Zone>";
		joiningRoom = true;
		pendingRoom = roomName;
		pendingZone = zoneName;
		inGame = false;
		send(action, parameters);
	}
	private function sendPublicMessage(message:String, variables:Object) {
		var action:String = "SendPublicMessage";
		//var parameters = "<Message><![CDATA["+message+"]]></Message><Variables><Variable><Name>Test Var</Name><Data>Yaya</Data></Variables>";
		var variableXML:String = "";
		for (var i in variables) {
			var value:String = variables[i].toString();
			variableXML += "<Variable><Name>"+i+"</Name><Data><![CDATA["+value+"]]></Data></Variable>";
		}
		if (variableXML != "") {
			variableXML = "<Variables>"+variableXML+"</Variables>";
		} else {
			variableXML = "<Variables />";
		}
		var parameters:String = "<Message><![CDATA["+message+"]]></Message>"+variableXML;
		send(action, parameters);
	}
	private function sendPrivateMessage(message:String, users:Array, variables:Object) {
		var action:String = "SendPrivateMessage";
		var usersXML:String = "<Users>";
		for (var i = 0; i<users.length; ++i) {
			var user:Object = users[i];
			usersXML += "<User>"+user+"</User>";
		}
		usersXML += "</Users>";
		var variableXML:String = "";
		for (var i in variables) {
			var value = variables[i];
			variableXML += "<Variable><Name>"+i+"</Name><Data><![CDATA["+value+"]]></Data></Variable>";
		}
		if (variableXML != "") {
			variableXML = "<Variables>"+variableXML+"</Variables>";
		} else {
			variableXML = "<Variables />";
		}
		var parameters:String = usersXML+"<Message>"+message+"</Message>"+variableXML;
		send(action, parameters);
	}
	private function connectionResponse(success:Boolean) {
		//This occurs when the basic connection is established with the socket-server
		ConnectedToServer = success;
		if (!ConnectedToServer) {
			//Could not find server or server is turned off
			connectionEstablished(false, "Could not establish a server connection");
		}
	}
	private function connectionEstablished(success:Boolean, reason:String) {
		//Sucessfully connection to ElectroServer is a two stage process.
		//First you must establish a connection. Then ElectroServer must tell you that it is ok.
		if (success && ConnectedToServer) {
			//connected successfully
			isConnected = true;
			onConnection(true);
		} else {
			//Did not connect successfully
			onConnection(false, reason);
		}
	}
	private function XMLReceived(info:XML) {
		var validXML:Boolean = info.toString().substr(0, 1) == "<";
		if (validXML) {
			parseXML(info);
		} else {
			rawMessageReceived(info.toString());
		}
	}
	private function parseXML(data:XML) {
		//hidden
		//Every packet of XML that is pushed in from the server hits this method forist
		if (getDebug()) {
			trace("-----incomming----");
			trace(data);
		}
		var info:Array = data.firstChild.childNodes;
		//Every XML packet has an action
		var action:String = info[0].firstChild.nodeValue;
		//Parse the parameters into easy-to-use data objects
		var parameters:Array = info[1].childNodes;
		var params:Object = new Object();
		parseParameters(parameters, params);
		//Apply the action
		applyAction(action, params, data);
	}
	private function isArrayNodeName(nodeName:String):Boolean {
		//hidden
		//The parseParameters function treats all nodes the same unless they are in
		// the 'nodeNameList', in which case they are converted to arrays
		//This function returns true|false depending on if the nodeName is in the list
		if (nodeNameList[nodeName]) {
			return true;
		} else {
			return false;
		}
	}
	private function terminateHere(xmls:XML):Boolean {
		//hidden
		//This is used by parseParameters to determine how deep the XML goes
		return xmls.childNodes[0].hasChildNodes() ? false : true;
	}
	private function send(action:String, parameters:String) {
		//hidden
		var message:String = "<XmlDoc><Action>"+action+"</Action><Parameters>"+parameters+"</Parameters></XmlDoc>";
		if (getDebug()) {
			trace("---out going----");
			trace(message);
		}
		super.send(message);
	}
	/*
		Sends a raw unchanged string to a plugin
		@description Used in conjunction with {@link #registerRawHandler}, this advanced method is used to send a raw message
					directly to ElectroServer. It won't be re-formatted by the ElectroServer class. It's just sent as-is. ElectroServer
					captures it and forwards it to the plugin specified when you used {@link #registerRawHandler}.
		@param The raw string to send.
		@example
			<code>
				es.sendRaw("t|23|f|90");
			</code>
	*/
	public function sendRaw(str:String) {
		super.send(str);
	}
	private function parseParameters(info:Array, parentOb:Object) {
		//hidden
		//This is a generic recursive method which walks the parameters node of the message
		//it parses the XML into generic data objects
		//====
		// example:
		// <Parameters><Results>Success</Results></Parameters>
		// parses to  parameters.results which is an object, the value of 'Results' is:
		// parameters.Results.value
		for (var i = 0; i<info.length; ++i) {
			var xmls:XML = info[i];
			var terminate:Boolean = terminateHere(xmls);
			var nodeName:String = xmls.nodeName;
			if (parentOb instanceof Array) {
				var ob:Object = new Object();
				parentOb.push(ob);
				var ob:Object = parentOb[parentOb.length-1];
			} else {
				parentOb[nodeName] = new Object();
				var ob:Object = parentOb[nodeName];
			}
			for (var j in xmls.attributes) {
				if (ob.attributes == undefined) {
					ob.attributes = new Object();
				}
				var value:String = xmls.attributes[j];
				if (!isNaN(Number(value))) {
					var value:Number = Number(value);
				}
				if (value.toLowerCase() == "true") {
					var value:Boolean = true;
				} else if (value.toLowerCase() == "false") {
					var value:Boolean = false;
				}
				ob.attributes[j] = value;
			}
			if (isArrayNodeName(nodeName)) {
				ob[nodeName] = new Array();
				var ob:Object = ob[nodeName];
			}
			if (terminate) {
				// No further sub-nodes
				var nodeValue:String = xmls.firstChild.nodeValue;
				if (!isNaN(Number(nodeValue)) && nodeValue != undefined && xmls.nodeName != "Message") {
					var nodeValue:Number = Number(nodeValue);
				}
				ob.value = nodeValue;
			} else {
				// There are sub-nodes that need parsing
				var nodeName:String = xmls.nodeName;
				var children:Array = xmls.childNodes;
				parseParameters(children, ob);
			}
		}
	}
	/*
		Logs you out of the server
		@description This method logs you out of ElectroServer. If the parameter is true then it also disconnects you from ElectroServer.
		@param Boolean value. If true then you are disconnected from ElectroServer too. If false, then your connection persists and you
				can re-login.
		@example <code>es.logout();</code>
	*/
	public function logout(disconnect:Boolean) {
		if (disconnect == undefined) {
			disconnect = true;
		}
		var action:String = "Logout";
		var parameters:String = "<Disconnect>"+disconnect+"</Disconnect>";
		send(action, parameters);
	}
	/*
		Logs you into the server
		@description Before you can use any of the server functionality you must login to the server. Using this method is
					how that is accomplished. If ElectroServer has not been configured to accept passwords, then the password
					field is not required.<br><br>
					The password field is used by the server in one of two ways:<br>
					1) To verify that you are a moderator for the server. A moderator has extended abilities such as kick and ban.<br>
					2) The password is passed through a custom-written login event handler for the server. This would be in place to verify login
					information against a database.<br><br>
					The login method returns nothing. However, once the server has either accepted or denied your login request the
					{@link #loggedIn} event is fired.
		@param The username to use
		@param The password to use. (Optional)
		@param An object containing variables to be sent to ElectroServer. This is useful when using a custom server-side login event handler. (Optional)
		@example 
				<code>
					function loginResponse(success:Boolean, error:String) {
						//This function is called when the server responds after a login request is sent
						if (success) {
							//properly logged in
							gotoAndStop("Chat")
						} else {
							//not properly logged in, there was an error
							msg.text = error;
							trace(error);
							gotoAndStop("Error")
						}
					}
					es.loggedIn = loginResponse;
					es.login("jobem");
				</code>
	*/
	public function login(tempUsername:String, tempPassword:String, variables:Object) {
		username = tempUsername;
		if (username == undefined || username == null) {
			username = "";
		}
		if (tempPassword == undefined) {
			password = "";
		} else {
			password = tempPassword;
		}
		//user.password = password;
		//username = user.username;
		//password = user.password;
		var action:String = "Login";
		var parameters:String = "<Name>"+username+"</Name><Password>"+password+"</Password>";
		if (variables != undefined) {
			parameters += "<EventHandlerVariables>";
			for (var i in variables) {
				parameters += "<EventHandlerVariable>";
				parameters += "<Name>"+i+"</Name>";
				parameters += "<Data>"+variables[i]+"</Data>";
				parameters += "</EventHandlerVariable>";
			}
			parameters += "</EventHandlerVariables>";
		}
		send(action, parameters);
	}
	/*
		Connects you to the server
		@description Creates a socket connection with ElectroServer using an IP and Port specified. Alternatively, the
						ip and port can be set using the {@link #setIP} and {@link #setPort} methods and left blank in this method.
						See also {@link #onConnection}.<br><br>
					<b>Explanation for beginners</b><br>
					ElectroServer is a socket-server, a piece of software written in Java that resides on a remote server 
					somewhere (or your own personal computer for testing). When the ElectroServer software is started 
					it "binds" itself to a specific IP address and "listens" for communication requests over a specific port.
					The connect() method is used to establish a connection with ElectroServer. By specifying the IP address and
					port that ElectroServer uses you can connect to it. This connection is called a persistent connection. It is
					persistent because the connection exists until it is closed by the Flash client, the server, or by killing
					the Flash client (leaving the web page).<br>
					While this connection exists data can be passed from the Flash client to the server or from the server
					to the Flash client without either side requesting the data. This data is intercepted and used by making
					use of many of the various methods and events of the ElectroServer object.<br>
					If you are truly a beginner and are having trouble understanding all of this, then please take a look at
					some open-source FLA examples that use the ElectroServer class.<br>
		@param The ip address of ElectroServer 
		@param The port over which ElectroServer is listening
		@example 
				Usage 1:
				<code>
					es.setIP("127.0.0.1");
					es.setPort(9875);
					es.connect();
				</code>
				Usage 2:
				<code>
					es.connect("127.0.0.1", 9875);
				</code>
	*/
	public function connect(tmp_ip:String, tmp_port:Number) {
		if (tmp_ip != undefined) {
			setIP(tmp_ip);
		}
		if (tmp_port != undefined) {
			setPort(tmp_port);
		}
		//Override the XMLSocket connect method
		onConnect = connectionResponse;
		onXML = XMLReceived;
		super.connect(getIP(), getPort());
	}
	/*
		Returns a reference to the object that represents you
		@description Every user in the userlist has an object that contains information about that user. This method
					returns the object that represents you. <br><br>
					<b>Properties on user object</b><br>
					<i>Name.value</i> The username<br>
					<i>userVariables</i> Uservariables associated with you. See {@link #createUserVariable}.<br>
					<i>AssignedNumber.value</i> If the room is numbering users, then this has your unique number in it.<br>
		@example 
				The following just traces your username:
				<code>
					trace(es.getUser().Name.value);
				</code>
				The following returns the number associated with you in a game.
				<code>
					var myNum:Number = es.getUser().AssignedNumber.value;
				</code>
		@returns Returns an object representing you.
	*/
	public function getUser() {
		return myUser;
	}
	private function applyAction(action:String, params:Object, originalXML:XML) {
		if (action == "ConnectionResponse") {
			//This is a willful response from the server
			var Major:Number = Number(params.Version.attributes.Major);
			var Minor:Number = Number(params.Version.attributes.Minor);
			var Sub:Number = Number(params.Version.attributes.Sub);
			if (params.Result.value == "Accepted" && ValidateVersion(Major, Minor, Sub)) {
				connectionEstablished(true);
			} else {
				//did not connect properly
				if (params.Result.value == "Accepted" && !ValidateVersion(Major, Minor, Sub)) {
					var error = "ElectroServer 3 version is too old. Please install latest.";
					trace("=====================================================");
					trace("===============Error Error Error=====================");
					trace("This class file is newer than the version of ElectroServer 3 that you are using.");
					trace("AS 2.0 Class version: "+MajorVersion+"."+MinorVersion+"."+SubVersion);
					trace("ElectroServer version: "+Major+"."+Minor+"."+Sub);
					trace("If it says NaN above, then you definately need to update.");
					trace("Visit http://www.electrotank.com/ElectroServer/");
					trace("=====================================================");
					trace("=====================================================");
				} else {
					var error = params.Reason.value;
				}
				connectionEstablished(false, error);
			}
		} else if (action == "LoginResponse") {
			if (params.Result.value.toLowerCase() == "accepted") {
				//var variables_array:Array = params.Variables.Variables;
				var variables:Array = params.Variables.Variables;
				var var_ob:Object = new Object();
				if (params.Username.value != undefined) {
					username = params.Username.value;
				}
				for (var j=0;j<variables.length;++j) {
					var ob = variables[j];
					var name = ob.Name.value;
					var value = ob.Data.value;
					var_ob[name] = value;
				}
				loggedIn(true, undefined, var_ob);
			} else {
				var error = params.Reason.value;
				loggedIn(false, error);
			}
		} else if (action == "GetAllZones") {
			var arr = params.Zones.Zones;
			var zones = new Array();
			for (var i=0;i<arr.length;++i) {
				var ob = arr[i]
				zones.push(ob.value)
			}
			allZonesLoaded(zones)
		} else if (action == "GetUserLocation") {
			var name = params.User.value
			var tmp_zone = params.Zone.value;
			var room = params.Room.value;
			if (params.Status.value == "Success") {
				userLocationLoaded(name, tmp_zone, room, true);
			} else {
				userLocationLoaded(name, tmp_zone, room, false);
			}
		} else if (action == "GetUsersInRoom") {
			var arr = params.Users.Users;
			var returnArr = new Array();
			for (var i=0;i<arr.length;++i) {
				var ob = new Object();
				ob.username = arr[i].value
				returnArr.push(ob);
			}
			usersInRoomLoaded(returnArr, params.Room.value, params.Zone.value);
		} else if (action == "FriendLoggedOut") {
			var name = params.FriendName.value;
			for (var i=0;i<buddyList.length;++i) {
				var ob = buddyList[i];
				if (ob.username == name) {
					ob.online = false;
					ob.status = "offline";
					ob.label = ob.username +" "+ob.status;
					break;
				}
			}
			buddyLoggedOut(name);
		} else if (action == "FriendLoggedIn") {
			var name = params.FriendName.value;
			for (var i = 0; i<buddyList.length; ++i) {
				var ob = buddyList[i];
				if (ob.username == name) {
					ob.online = true;
					ob.status = "online";
					ob.label = ob.username+" "+ob.status;
					break;
				}
			}
			buddyLoggedIn(name);
		} else if (action == "CreateRoom") {
			if (params.Result.value == "Failure") {
				var error = params.Reason.value;
				if (error.indexOf("already exists")>-1 && auto_join) {
					joinRoom(pendingRoom, "", pendingZone);
				}
			} else {
				//Success, do nothing
			}
		} else if (action == "LoadConfiguration") {
			//This is for admin
			var num_packets:String = params.NumberOfPackets.value;
			var num:String = params.PacketNumber.value;
			var packetData:String = params.PacketData.value;
			if (num == "1") {
				configXML = packetData;
			} else {
				configXML += packetData;
			}
			if (num == num_packets) {
				parseConfig(configXML);
			}
		} else if (action == "UpdateUserVariable") {
			var users = zone.users;
			var ob = params.UserVariable;
			var tmp_user = ob.User.value;
			var value = ob.Data.value;
			var name = ob.Name.value;
			for (var i = 0; i<users.length; ++i) {
				var tmp = users[i].Name.value;
				if (tmp_user == tmp) {
					var user = users[i];
					var type = typeof user.userVariables[name];
					if (type.toString() == "undefined") {
						if (typeof user.userVariables.toString() == "undefined") {
							user.userVariables = new Object();
						}
						user.userVariables[name] = value;
						userVariableUpdated(user, "created", name);
					} else {
						if (value.toString() == "null") {
							delete user.userVariables[name];
							userVariableUpdated(user, "deleted", name);
						} else {
							user.userVariables[name] = value;
							userVariableUpdated(user, "updated", name);
						}
					}
					break;
				}
			}
		} else if (action == "JoinRoom") {
			if (params.Result.value == "Success") {
				//joined the room successfully
				var roomName:String = pendingRoom;
				var roomList:Array = zone.rooms;
				var found:Boolean = false;
				for (var i = 0; i<roomList.length; ++i) {
					var roomOb:Object = roomList[i];
					if (roomOb.Name.value == roomName) {
						found = true;
						zone.myRoom = roomOb;
						var rv = params.RoomVariables.RoomVariables;
						zone.myRoom.roomVariables = new Object();
						for (var j = 0; j<rv.length; ++j) {
							var name = rv[j].Name.value;
							var value = rv[j].Data.value;
							zone.myRoom.roomVariables[name] = value;
						}
						roomVariablesUpdated("all", zone.myRoom.roomVariables);
						break;
					}
				}
				if (!found) {
					zone.myRoom = new Object();
					zone.Name = new Object();
					zone.Name.value = pendingZone;
					zone.myRoom.Name = new Object();
					zone.myRoom.Name.value = pendingRoom;
				}
				var rv = params.RoomVariables.RoomVariables;
				zone.myRoom.roomVariables = new Object();
				for (var j = 0; j<rv.length; ++j) {
					var name = rv[j].Name.value;
					var value = rv[j].Data.value;
					zone.myRoom.roomVariables[name] = value;
				}
				roomVariablesUpdated("all", zone.myRoom.roomVariables);
				roomJoined({success:true}, zone.myRoom);
				delete pendingRoom;
				joiningRoom = false;
				//deal with userlist
				zone.users = params.Users.Users;
				for (var i = 0; i<zone.users.length; ++i) {
					var user:Object = zone.users[i];
					user.label = user.Name.value;
					if (showModeratorLabel && user.attributes.Moderator) {
						user.label += moderatorLabel;
					}
					user.data = user;
					var uv = user.UserVariables.UserVariables;
					user.userVariables = new Object();
					for (var k = 0; k<uv.length; ++k) {
						var u = uv[k];
						var user_var_name = u.Name.value;
						var user_var_val = u.Data.value;
						user.userVariables[user_var_name] = user_var_val;
					}
					if (user.Name.value == username) {
						myUser = user;
						if (user.AssignedNumber.value == 0) {
							user.isGameMaster = true;
						}
					}
				}
				userListUpdated(zone.users, "all");
			} else {
				//Failed to join the room
				var error = params.Reason.value;
				roomJoined({success:false, error:error});
			}
		} else if (action == "RenumberUsers") {
			var users = zone.users;
			for (var i = 0; i<params.Users.Users.length; ++i) {
				var uname:String = params.Users.Users[i].Name.value;
				for (var j = 0; j<users.length; ++j) {
					if (uname == users[j].Name.value) {
						users[j].AssignedNumber.value = params.Users.Users[i].AssignedNumber.value;
						users[j].isGameMaster = false;
					}
				}
				if (getUser().AssignedNumber.value == 0) {
					getUser().isGameMaster = true;
				}
			}
			if (getUser().AssignedNumber.value == 0) {
				isGameMaster = true;
			} else {
				isGameMaster = false;
			}
			usersRenumbered(users);
		} else if (action == "SendPrivateMessage") {
			var message = params.Message.value;
			var from = params.User.value;
			var variables = new Object();
			for (var i = 0; i<params.Variables.Variables.length; ++i) {
				var name = params.Variables.Variables[i].Name.value;
				var value = params.Variables.Variables[i].Data.value;
				variables[name] = value;
			}
			if (variables.Action.toLowerCase() == "move") {
				var what:String = message;
				var foo:Wddx = new Wddx();
				var ob:Object = foo.deserialize(what);
				moveReceived("private", ob, from);
			} else {
				messageReceived("private", message, from, variables);
			}
		} else if (action == "SendPublicMessage") {
			var message:String = params.Message.value;
			var from:String = params.User.value;
			var variables:Object = new Object();
			for (var i = 0; i<params.Variables.Variables.length; ++i) {
				var name:String = params.Variables.Variables[i].Name.value;
				var value = params.Variables.Variables[i].Data.value;
				variables[name] = value;
			}
			if (variables.Action.toLowerCase() == "move") {
				var what:String = message;
				var foo:Wddx = new Wddx();
				var ob:Object = foo.deserialize(what);
				moveReceived("public", ob, from);
			} else {
				messageReceived("public", message, from, variables);
			}
		} else if (action == "GetRoomsInZone") {
			var tmp_rooms = params.Zone.Rooms.Rooms;
			var show_nums = true;
			var roomsToReturn = new Array();
			for (var i = 0; i<tmp_rooms.length; ++i) {
				var room_name = tmp_rooms[i].Name.value;
				var label:String = "";
				if (show_nums) {
					label = room_name+" ("+tmp_rooms[i].attributes.Users+")";
				} else {
					label = room_name;
				}
				tmp_rooms[i].label = label;
				tmp_rooms[i].iconAlignment = "right";
				var WDDXxml:String = tmp_rooms[i].Description.value;
				if (WDDXxml.length>2) {
					var foo:Wddx = new Wddx();
					var ob:Object = foo.deserialize(WDDXxml);
				} else {
					var ob:Object = new Object();
				}
				tmp_rooms[i].Description = ob;
				//var room:Object = zone.rooms[i];
				//room.data = room;
			}
			onRoomsInZoneLoaded(tmp_rooms, params.Zone.Name.value);
		} else if (action == "RoomList") {
			var myOldRoom = getRoom();
			var oldUsers = getUserList();
			zone = new Object();
			zone.numUsers = Number(params.Zone.attributes.Users);
			zone.name = params.Zone.Name.value;
			zone.rooms = params.Zone.Rooms.Rooms;
			for (var i = 0; i<zone.rooms.length; ++i) {
				var WDDXxml:String = zone.rooms[i].Description.value;
				if (WDDXxml.length>2) {
					var foo:Wddx = new Wddx();
					var ob:Object = foo.deserialize(WDDXxml);
				} else {
					var ob:Object = new Object();
				}
				zone.rooms[i].Description = ob;
				var room:Object = zone.rooms[i];
				room.label = room.Name.value;
				room.data = room;
				if (myOldRoom.Name.value == room.Name.value) {
					zone.rooms[i] = myOldRoom;
					zone.myRoom = myOldRoom;
					zone.users = oldUsers;
				}
			}
			zoneChanged(zone.name);
			roomListUpdated(zone.rooms, "all");
		} else if (action == "UpdateUserList") {
			var minorAction:String = params.MinorAction.value;
			if (minorAction == "UserJoined") {
				var users = zone.users;
				var user = params.User;
				//
				var user_name = user.Name.value;
				var uv = user.UserVariables.UserVariables;
				user.userVariables = new Object();
				for (var k = 0; k<uv.length; ++k) {
					var u = uv[k];
					var user_var_name = u.Name.value;
					var user_var_val = u.Data.value;
					user.userVariables[user_var_name] = user_var_val;
				}

				//
				user.label = user.Name.value;
				if (showModeratorLabel && user.attributes.Moderator) {
					user.label += moderatorLabel;
				}
				user.data = user;
				users.push(user);
				userListUpdated(zone.users, "userjoined", user.Name.value, user);
			} else if (minorAction == "UserLeft") {
				var users = zone.users;
				var user = params.User;
				var uname = user.Name.value;
				for (var i = 0; i<users.length; ++i) {
					var name = users[i].Name.value;
					if (uname == name) {
						users.splice(i, 1);
						break;
					}
				}
				userListUpdated(zone.users, "userleft", user.Name.value);
			}
		} else if (action == "SendPluginMessage") {
			var plugin:String = params.User.value;
			var pluginAction:String = params.Message.value;
			var variables:Array = params.Variables.Variables;
			var ob:Object = new Object();
			for (var i = 0; i<variables.length; ++i) {
				var name:String = variables[i].Name.value;
				var value = variables[i].Value.value;
				ob[name] = value;
			}
			pluginMessageReceived(plugin, pluginAction, ob);
		} else if (action == "UpdateRoomVariable") {
			var minorAction:String = params.MinorAction.value;
			if (minorAction == "Create") {
				var name:String = params.RoomVariable.Name.value;
				var value = params.RoomVariable.Data.value;
				zone.myRoom.roomVariables[name] = value;
				roomVariablesUpdated("created", zone.myRoom.roomVariables, name);
			} else if (minorAction == "Update") {
				var name:String = params.RoomVariable.Name.value;
				var value = params.RoomVariable.Data.value;
				zone.myRoom.roomVariables[name] = value;
				roomVariablesUpdated("updated", zone.myRoom.roomVariables, name);
			} else if (minorAction == "Delete") {
				var name:String = params.RoomVariable.Name.value;
				delete zone.myRoom.roomVariables[name];
				roomVariablesUpdated("deleted", zone.myRoom.roomVariables, name);
			}
		} else if (action == "GetLoggedInUserCount") {
			var num = Number(params.LoggedInUsers.value);
			loggedInUserCountUpdated(num);
		} else if (action == "GetServerTime") {
			var num = Number(params.ServerTime.value);
			onGetServerTime(num);
		} else if (action == "UpdateRoomList") {
			var minorAction:String = params.MinorAction.value;
			zone.numUsers = Number(params.Zone.attributes.Users);
			zoneUpdated(zone.numUsers);
			var newRoomOb:Object = params.Zone.Rooms.Rooms[0];
			if (minorAction == "ChangeRoomDetails") {
				//Room that already exists was changed
				for (var j = 0; j<zone.rooms.length; ++j) {
					var roomOb:Object = zone.rooms[j];
					if (roomOb.Name.value == newRoomOb.Name.value) {
						roomOb.label = roomOb.Name.value;
						roomOb.data = roomOb;
						for (var i in newRoomOb) {
							roomOb[i] = newRoomOb[i];
						}
						var WDDXxml:String = newRoomOb.Description.value;
						if (WDDXxml.length>2) {
							var foo:Wddx = new Wddx();
							var ob:Object = foo.deserialize(WDDXxml);
						} else {
							var ob:Object = new Object();
						}
						roomOb.Description = ob;
						break;
					}
				}
				roomListUpdated(zone.rooms, "roomupdated", roomOb);
			} else if (minorAction == "CreateRoom") {
				//Add a new room to the room list
				var roomOb:Object = newRoomOb;
				roomOb.label = roomOb.Name.value;
				roomOb.data = roomOb;
				var WDDXxml:XML = roomOb.Description.value;
				var ob:Object;
				if (WDDXxml != undefined && WDDXxml != "") {
					var foo:Wddx = new Wddx();
					ob = foo.deserialize(WDDXxml);
				} else {
					ob = new Object();
				}
				roomOb.Description = ob;
				zone.rooms.push(roomOb);
				roomListUpdated(zone.rooms, "roomcreated", roomOb);
			} else if (minorAction == "DeleteRoom") {
				//Room that already exists was deleted
				for (var j = 0; j<zone.rooms.length; ++j) {
					var roomOb:Object = zone.rooms[j];
					if (roomOb.Name.value == newRoomOb.Name.value) {
						var deleteIt = true;
						var name = newRoomOb.Name.value;
						if (name == getRoom().Name.value && !joiningRoom) {
							deleteIt = false;
						}
						if (deleteIt) {
							zone.rooms.splice(j, 1);
						}
						break;
					}
				}
				roomListUpdated(zone.rooms, "RoomDeleted", name);
			}
		}
	}
}
